searchData={"content_type":"text/markdown","items":[{"doc":"Coherence is a full featured, configurable authentication and user\nmanagement system for Phoenix, providing a number of optional features\nconfigured with a installation mix task:\n\n* Database Authenticatable: handles hashing and storing an encrypted password in the database.\n* Invitable: sends invites to new users with a sign-up link, allowing the user to create their account with their own password.\n* Registerable: allows anonymous users to register a users email address and password.\n* Confirmable: new accounts require clicking a link in a confirmation email.\n* Recoverable: provides a link to generate a password reset link with token expiry.\n* Trackable: saves login statics like login counts, timestamps, and IP address for each user.\n* Lockable: locks an account when a specified number of failed sign-in attempts has been exceeded.\n* Unlockable With Token: provides a link to send yourself an unlock email.\n* Rememberable: provides persistent login with 'Remember me?' check box on login page.\n\nSee the [README](readme.html) file for an overview, installation, and\nsetup instructions.\n\n#","ref":"Coherence.html","title":"Coherence","type":"module"},{"doc":"Handles hashing and storing an encrypted password in the database.\n\nProvides `/sessions/new` and `/sessions/delete` routes for logging in and out with\nthe appropriate templates and view.\n\nThe following columns are added the ` _add_coherence_to_user.exs` migration:\n\n* :password_hash, :string - the encrypted password\n\nThe following options can be customized (default shown):\n\n    config :coherence,\n      password_hash_field: :password_hash    # you must recompile Coherence after changing this\n\n\n#","ref":"Coherence.html#module-authenticatable","title":"Authenticatable - Coherence","type":"module"},{"doc":"Handles sending invites to new users with a sign-up link, allowing the user to create their account with their own password.\n\nProvides `/invitations/new` and `invitations/edit` routes for creating a new invitation and creating a new account from the invite email.\n\nThese routes can be configured to require login by using the `coherence_routes :protected` macro in your router.exs file.\n\nInvitation token timeout will be added in the future.\n\nThe following table is created by the generated ` _create_coherence_invitable.exs` migration:\n\n    create table(:invitations) do\n      add :name, :string\n      add :email, :string\n      add :token, :string\n    end\n\n#","ref":"Coherence.html#module-invitable","title":"Invitable - Coherence","type":"module"},{"doc":"Allows anonymous users to register a users email address and password.\n\nProvides `/registrations/new` and `/registrations/create` routes for creating a new registration.\n\nAdds a `Register New Account` to the log-in page.\n\nIt is recommended that the :confirmable option is used with :registerable to\nensure a valid email address is captured.\n\n#","ref":"Coherence.html#module-registerable","title":"Registerable - Coherence","type":"module"},{"doc":"Requires a new account be conformed. During registration, a confirmation token is generated and sent to the registering email. This link must be clicked before the user can sign-in.\n\nProvides `edit` action for the `/confirmations` route.\n\nThe following options can be customized (default shown):\n\n    config :coherence,\n      confirmation_token_expire_days: 5,\n      allow_unconfirmed_access_for: 0       # number of days. 0 to disable\n\n#","ref":"Coherence.html#module-confirmable","title":"Confirmable - Coherence","type":"module"},{"doc":"Allows users to reset their password using an expiring token send by email.\n\nProvides `new`, `create`, `edit`, `update` actions for the `/passwords` route.\n\nAdds a \"Forgot your password?\" link to the log-in form. When clicked, the user provides their email address and if found, sends a reset password instructions email with a reset link.\n\nThe following options can be customized (default shown):\n\n    config :coherence,\n      reset_token_expire_days: 2\n\n#","ref":"Coherence.html#module-recoverable","title":"Recoverable - Coherence","type":"module"},{"doc":"Saves login statics like login counts, timestamps, and IP address for each user.\n\nAdds the following database field to your User model with the generated migration:\n\n    add :sign_in_count, :integer, default: 0  # how many times the user has logged in\n    add :current_sign_in_at, :datetime        # the current login timestamp\n    add :last_sign_in_at, :datetime           # the timestamp of the previous login\n    add :current_sign_in_ip, :string          # the current login IP adddress\n    add :last_sign_in_ip, :string             # the IP address of the previous login\n\n#","ref":"Coherence.html#module-trackable","title":"Trackable - Coherence","type":"module"},{"doc":"Locks an account when a specified number of failed sign-in attempts has been exceeded.\n\nThe following defaults can be changed with the following config entries:\n\nThe following options can be customized (default shown):\n\n    config :coherence,\n      unlock_timeout_minutes: 5,\n      max_failed_login_attempts: 5\n\nAdds the following database field to your User model with the generated migration:\n\n    add :failed_attempts, :integer, default: 0\n    add :unlock_token, :string\n    add :locked_at, :datetime\n\n#","ref":"Coherence.html#module-lockable","title":"Lockable - Coherence","type":"module"},{"doc":"Provides a link to send yourself an unlock email. When the user clicks the link, the user is presented a form to enter their email address and password. If the token has not expired and the email and password are valid, a unlock email is sent to the user's email address with an expiring token.\n\nThe following options can be customized (default shown):\n\n    config :coherence,\n      unlock_token_expire_minutes: 5\n\n#","ref":"Coherence.html#module-unlockable-with-token","title":"Unlockable with Token - Coherence","type":"module"},{"doc":"The `rememberable` option provides persistent login when the 'Remember Me?' box is checked during login.\n\nWith this feature, you will automatically be logged in from the same browser when your current login session dies using a configurable expiring persistent cookie.\n\nFor security, both a token and series number stored in the cookie on initial login. Each new creates a new token, but preserves the series number, providing protection against fraud. As well, both the token and series numbers are hashed before saving them to the database, providing protection if the database is compromised.\n\nThe following options can be customized (default shown):\n\n    config :coherence,\n      rememberable_cookie_expire_hours: 2*24,\n      login_cookie: \"coherence_login\"\n\nThe following table is created by the generated ` _create_coherence_rememberable.exs` migration:\n\n    create table(:rememberables) do\n      add :series_hash, :string\n      add :token_hash, :string\n      add :token_created_at, :datetime\n      add :user_id, references(:users, on_delete: :delete_all)\n\n      timestamps\n    end\n    create index(:rememberables, [:user_id])\n    create index(:rememberables, [:series_hash])\n    create index(:rememberables, [:token_hash])\n    create unique_index(:rememberables, [:user_id, :series_hash, :token_hash])\n\nThe `--rememberable` install option is not provided in any of the installer group options. You must provide the `--rememberable` option to install the migration and its support.","ref":"Coherence.html#module-remember-me","title":"Remember Me - Coherence","type":"module"},{"doc":"Backwords compatibility note: For versions of Phoenix previous to 1.3 use `mix coherence. `. The new `mix coh. ` better understands the newer Phoenix patterns.\n\n#","ref":"Coherence.html#module-mix-tasks","title":"Mix Tasks - Coherence","type":"module"},{"doc":"The following examples illustrate various configuration scenarios for the install mix task:\n\n    # Install with only the `authenticatable` option\n    $ mix coh.install\n\n    # Install all the options except `confirmable` and `invitable`\n    $ mix coh.install --full\n\n    # Install all the options except `invitable`\n    $ mix coh.install --full-confirmable\n\n    # Install all the options except `confirmable`\n    $ mix coh.install --full-invitable\n\n    # Install the `full` options except `lockable` and `trackable`\n    $ mix coh.install --full --no-lockable --no-trackable\n\nAnd some reinstall examples:\n\n    # Reinstall with defaults (--silent --no-migrations --no-config --confirm-once)\n    $ mix coh.install --reinstall\n\n    # Confirm to overwrite files, show instructions, and generate migrations\n    $ mix coh.install --reinstall --no-confirm-once --with-migrations\n\nRun `$ mix help coh.install` for more information.\n\n#","ref":"Coherence.html#module-installer","title":"Installer - Coherence","type":"module"},{"doc":"The following examples illustrate how to remove the files created by the installer:\n\n    # Clean all the installed files\n    $ mix coh.clean --all\n\n    # Clean only the installed view and template files\n    $ mix coh.clean --views --templates\n\n    # Clean all but the models\n    $ mix coh.clean --all --no-models\n\n    # Prompt once to confirm the removal\n    $ mix coh.clean --all --confirm-once\n\nAfter installation, if you later want to remove one more options, here are a couple examples:\n\n    # Clean one option\n    $ mix coh.clean --options=recoverable\n\n    # Clean several options without confirmation\n    $ mix coh.clicked --no-confirm --options=\"recoverable unlockable-with-token\"\n\n    # Test the uninstaller without removing files\n    $ mix coh.clicked --dry-run --options=\"recoverable unlockable-with-token\"\n\nRun `$ mix help coh.install` or `$ mix help coherence.install` for more information.","ref":"Coherence.html#module-clean","title":"Clean - Coherence","type":"module"},{"doc":"Get the currently logged in user data.","ref":"Coherence.html#current_user/1","title":"Coherence.current_user/1","type":"function"},{"doc":"Get the currently logged in user name.","ref":"Coherence.html#current_user_name/2","title":"Coherence.current_user_name/2","type":"function"},{"doc":"Check if user is logged in.","ref":"Coherence.html#logged_in?/1","title":"Coherence.logged_in?/1","type":"function"},{"doc":"Updates the user login data in the current sessions.\n\nOther sessions belonging to the same user won't be updated.\nRequires access to the `conn`, which means it can't be called outside of the context of a conn.\nTo update all session belonging to the user see `update_user_login/1`.","ref":"Coherence.html#update_user_login/2","title":"Coherence.update_user_login/2","type":"function"},{"doc":"Updates the user login data in all the sessions belonging to the user.\n\nAll sessions belonging to the same user will be updated.\nDoesn't need access to the `conn`, which means it can be called anywhere.\nTo update only the current session see `update_user_login/2`","ref":"Coherence.html#update_user_logins/1","title":"Coherence.update_user_logins/1","type":"function"},{"doc":"Get the currently assigned user_token","ref":"Coherence.html#user_token/1","title":"Coherence.user_token/1","type":"function"},{"doc":"Verify a user token for channel authentication.","ref":"Coherence.html#verify_user_token/3","title":"Coherence.verify_user_token/3","type":"function"},{"doc":"Implements basic HTTP authentication. To use add:\n\n    plug Coherence.Authentication.Basic, realm: \"Secret world\"\n\n  to your pipeline.\n\n  This module is derived from https://github.com/bitgamma/plug_auth which is derived from https://github.com/lexmag/blaguth","ref":"Coherence.Authentication.Basic.html","title":"Coherence.Authentication.Basic","type":"module"},{"doc":"","ref":"Coherence.Authentication.Basic.html#call/2","title":"Coherence.Authentication.Basic.call/2","type":"function"},{"doc":"","ref":"Coherence.Authentication.Basic.html#create_login/4","title":"Coherence.Authentication.Basic.create_login/4","type":"function"},{"doc":"Returns the encoded form for the given `user` and `password` combination.","ref":"Coherence.Authentication.Basic.html#encode_credentials/2","title":"Coherence.Authentication.Basic.encode_credentials/2","type":"function"},{"doc":"","ref":"Coherence.Authentication.Basic.html#init/1","title":"Coherence.Authentication.Basic.init/1","type":"function"},{"doc":"Update login store for a user. `user_data` can be any term but must not be `nil`.","ref":"Coherence.Authentication.Basic.html#update_login/4","title":"Coherence.Authentication.Basic.update_login/4","type":"function"},{"doc":"","ref":"Coherence.Authentication.Basic.html#t:conn/0","title":"Coherence.Authentication.Basic.conn/0","type":"type"},{"doc":"","ref":"Coherence.Authentication.Basic.html#t:t/0","title":"Coherence.Authentication.Basic.t/0","type":"type"},{"doc":"Implements ip address based authentication. To use add\n\n    plug Coherence.Authentication.IpAddress, allow: ~w(127.0.0.1 192.168.1.200)\n\n  to your pipeline.\n\nIP addresses can be specified in a list as either IP or IP/subnet_mask, where subnet_mask\ncan be an integer or dot format.\n\nIf you would like access to the current user you must set each authorized IP address like:\n\n    Coherence.CredentialStore.Server.put_credentials({127.0.0.1}, %{role: :admin})\n\nor use a custom store like:\n\n    defmodule MyProject.Store do\n      def get_user_data(ip) do\n        Repo.one from u in User, where: u.ip_address == ^id\n      end\n    end\n\n    plug Coherence.Authentication.IpAddress, allow: ~w(127.0.0.1 192.168.1.0/24), store: &MyProject.Store/1","ref":"Coherence.Authentication.IpAddress.html","title":"Coherence.Authentication.IpAddress","type":"module"},{"doc":"allow: ~w(127.0.0.1 192.169.1.0/255.255.255.0)\n    allow: ~w(127.0.0.1 192.169.1.0/24)\n    deny: ~w(10.10.0.0/16)","ref":"Coherence.Authentication.IpAddress.html#module-ip-format-examples","title":"IP Format Examples: - Coherence.Authentication.IpAddress","type":"module"},{"doc":"* `:allow` - list of allowed IPs\n* `:deny` - list of denied IPs\n* `:error` - error to be displayed if the IP is not allowed\n* `:store` - the user_data store\n* `:assign_key` - the assigns key to store the user_data","ref":"Coherence.Authentication.IpAddress.html#module-options","title":"Options - Coherence.Authentication.IpAddress","type":"module"},{"doc":"Add the credentials for a `token`. `user_data` can be any term but must not be `nil`.","ref":"Coherence.Authentication.IpAddress.html#add_credentials/3","title":"Coherence.Authentication.IpAddress.add_credentials/3","type":"function"},{"doc":"","ref":"Coherence.Authentication.IpAddress.html#call/2","title":"Coherence.Authentication.IpAddress.call/2","type":"function"},{"doc":"","ref":"Coherence.Authentication.IpAddress.html#init/1","title":"Coherence.Authentication.IpAddress.init/1","type":"function"},{"doc":"Remove the credentials for a `token`.","ref":"Coherence.Authentication.IpAddress.html#remove_credentials/2","title":"Coherence.Authentication.IpAddress.remove_credentials/2","type":"function"},{"doc":"","ref":"Coherence.Authentication.IpAddress.html#t:conn/0","title":"Coherence.Authentication.IpAddress.conn/0","type":"type"},{"doc":"","ref":"Coherence.Authentication.IpAddress.html#t:t/0","title":"Coherence.Authentication.IpAddress.t/0","type":"type"},{"doc":"Implements Session based authentication. By default, it uses an Server for\nsession state. Additionally, a the session can be stored in a database with\nan Server based cache.\n\nThe plug can be used to force a login for unauthenticated users for routes\nthat need to be protected with a password.\n\nFor example:\n\n    plug Coherence.Authentication.Session, protected: true\n\nwill present the user for a login if they are accessing a route or controller\nthat uses this plug.\n\nFor pages that don't require authorization but would like to present logged in\ninformation on unprotected pages, use the default:\n\n    plug Coherence.Authentication.Session\n\nThis will set the current_user for use in templates, but not allow access to\nprotected pages.\n\nBy default, the user model for a logged-in user can be accessed with\n`Coherence.current_user(conn)` which is just a shortcut for\n`conn.assigns[:current_user]`. This can be changed with the global\n:assigns_key config option.","ref":"Coherence.Authentication.Session.html","title":"Coherence.Authentication.Session","type":"module"},{"doc":"This plug can be used in either the router.ex file or in a controller file.","ref":"Coherence.Authentication.Session.html#module-controller-based-authentication","title":"Controller Based Authentication - Coherence.Authentication.Session","type":"module"},{"doc":"To enable database persistence, implement [Coherence.DbStore] protocol for your\nuser model. As well, you will need to provide the :db_model option to the plug.\n\nSee the full guide on how to implement this in the project wiki:\nhttps://github.com/smpallen99/coherence/wiki/Session-token-Ecto-Persistance\n\nYou should be aware that the Server is still used to fetch the user data if can\nbe found. If the key is not found, it checks the database. If a record is found\nin the database, the server is updated and the user data returned.\n\n\nThis module is derived from https://github.com/lexmag/blaguth","ref":"Coherence.Authentication.Session.html#module-database-persistence","title":"Database Persistence - Coherence.Authentication.Session","type":"module"},{"doc":"Create a login for a user. `user_data` can be any term but must not be `nil`.","ref":"Coherence.Authentication.Session.html#create_login/3","title":"Coherence.Authentication.Session.create_login/3","type":"function"},{"doc":"Delete a login.","ref":"Coherence.Authentication.Session.html#delete_login/2","title":"Coherence.Authentication.Session.delete_login/2","type":"function"},{"doc":"Update login store for a user. `user_data` can be any term but must not be `nil`.","ref":"Coherence.Authentication.Session.html#update_login/3","title":"Coherence.Authentication.Session.update_login/3","type":"function"},{"doc":"","ref":"Coherence.Authentication.Session.html#t:conn/0","title":"Coherence.Authentication.Session.conn/0","type":"type"},{"doc":"","ref":"Coherence.Authentication.Session.html#t:t/0","title":"Coherence.Authentication.Session.t/0","type":"type"},{"doc":"Implements token based authentication. To use add\n\n    plug Coherence.Authentication.Token, source: :params, param: \"auth_token\"\n\n  or\n\n    plug Coherence.Authentication.Token, source: :session, param: \"auth_token\"\n\n  or\n\n    plug Coherence.Authentication.Token, source: :header, param: \"x-auth-token\"\n\n  or\n\n    plug Coherence.Authentication.Token, source: { module, function, [\"my_param\"]} end\n\n  or\n\n    plug Coherence.Authentication.Token, source: :params_session, param: \"auth_token\"\n\n  to your pipeline.","ref":"Coherence.Authentication.Token.html","title":"Coherence.Authentication.Token","type":"module"},{"doc":"* `source` - where to locate the token\n  * `error` - The error message if not authenticated\n  * `assigns_key` - The key to user in assigns (:current_uer)\n  * `store` - Where to store the token data","ref":"Coherence.Authentication.Token.html#module-options","title":"Options - Coherence.Authentication.Token","type":"module"},{"doc":"Add the credentials for a `token`. `user_data` can be any term but must not be `nil`.","ref":"Coherence.Authentication.Token.html#add_credentials/3","title":"Coherence.Authentication.Token.add_credentials/3","type":"function"},{"doc":"","ref":"Coherence.Authentication.Token.html#call/2","title":"Coherence.Authentication.Token.call/2","type":"function"},{"doc":"","ref":"Coherence.Authentication.Token.html#check_token_from_session/2","title":"Coherence.Authentication.Token.check_token_from_session/2","type":"function"},{"doc":"Utility function to generate a random authentication token.","ref":"Coherence.Authentication.Token.html#generate_token/0","title":"Coherence.Authentication.Token.generate_token/0","type":"function"},{"doc":"","ref":"Coherence.Authentication.Token.html#get_token_from_header/2","title":"Coherence.Authentication.Token.get_token_from_header/2","type":"function"},{"doc":"","ref":"Coherence.Authentication.Token.html#get_token_from_params/2","title":"Coherence.Authentication.Token.get_token_from_params/2","type":"function"},{"doc":"","ref":"Coherence.Authentication.Token.html#get_token_from_params_session/2","title":"Coherence.Authentication.Token.get_token_from_params_session/2","type":"function"},{"doc":"","ref":"Coherence.Authentication.Token.html#get_token_from_session/2","title":"Coherence.Authentication.Token.get_token_from_session/2","type":"function"},{"doc":"","ref":"Coherence.Authentication.Token.html#init/1","title":"Coherence.Authentication.Token.init/1","type":"function"},{"doc":"Remove the credentials for a `token`.","ref":"Coherence.Authentication.Token.html#remove_credentials/2","title":"Coherence.Authentication.Token.remove_credentials/2","type":"function"},{"doc":"","ref":"Coherence.Authentication.Token.html#save_token_in_session/2","title":"Coherence.Authentication.Token.save_token_in_session/2","type":"function"},{"doc":"","ref":"Coherence.Authentication.Token.html#t:conn/0","title":"Coherence.Authentication.Token.conn/0","type":"type"},{"doc":"","ref":"Coherence.Authentication.Token.html#t:t/0","title":"Coherence.Authentication.Token.t/0","type":"type"},{"doc":"Coherence Configuration Module.\n\nProvides a small wrapper around `Application.get_env :coherence`, providing an\naccessor function for each configuration items.\n\nConfiguration items can be defined as either a single atom or {name, default}\ntuple. Each of the items can be included in your `config/config.exs` file.\n\nThe following items are supported:\n\n* :allow_silent_password_recovery_for_unknown_user (false)\n* :allow_unconfirmed_access_for (0) - default 0 days\n* :assigns_key (:current_user)\n* :async_rememberable? (false) - Don't update rememberable seq_no for ajax requests\n* :auth_module (Coherence.Authentication.Session)\n* :changeset - Custom user changeset\n* :confirm_email_updates (false)  - All email updates should be confirmed by email (using the unconfirmed_email field)\n* :confirmation_token_expire_days (5)\n* :create_login (:create_login)\n* :credential_store - override the credential store module\n* :delete_login (:delete_login)\n* :email_from  - Deprecated. Use `email_from_name` and `email_from_email` instead\n* :email_from_email\n* :email_from_name\n* :email_reply_to - Set to true to add email_from_name and email_from_email\n* :email_reply_to_email\n* :email_reply_to_name\n* :invitation_permitted_attributes - List of allowed invitation parameter attribues as strings\n* :layout - Customize the layout template e.g. {MyApp.LayoutView, \"app.html\"}\n* :log_emails - Set to true to log each rendered email.\n* :logged_in_url\n* :logged_out_url\n* :login_cookie (\"coherence_login\")                 - The name of the login cookie\n* :login_field (:email) - The user model field used to login\n* :max_failed_login_attempts (5)\n* :messages_backend - (MyApp.Coherence.Messages)\n* :minimum_password_length The minimum password length to be accepted. Default value is 4.\n* :module - the name of project module (`module: MyProject`)\n* :opts ([])\n* :password_hash_field (:password_hash) - The field used to save the hashed password\n* :password_hashing_alg (Comeonin.Bcrypt) - Password hashing algorithm to use.\n* :password_reset_permitted_attributes - List of allowed password reset atributes as stings,\n* :registration_permitted_attributes - List of allowed registration parameter attributes as strings\n* :repo: the module name of your Repo (`repo: MyProject.Repo`)\n* :rememberable_cookie_expire_hours (2*24)\n* :reset_token_expire_days (2)\n* :router: the module name of your Router (`router: MyProject.Router`)\n* :schema_key\n* :session_key (\"session_auth\")\n* :session_permitted_attributes - List of allowed session attributes as strings\n* :site_name                                        - The site name used for email\n* :title  - Layout page title\n* :token_assigns_key (:user_token) - key used to access the channel_token in the conn.assigns map\n* :token_generator   (fn conn, user -> Phoenix.Token.sign(conn, \"user socket\", user.id) end) - override the default\n  may also provide an arity 3 function as a tuple {Module, :function, args}\n  where apply(Module, function, args) will be used\n* :token_max_age (2 * 7 * 24 * 60 * 60) - Phoenix.Token max_age\n* :token_salt (\"user socket\") - Phoenix.Token salt\n* :update_login (:update_login)\n* :unlock_timeout_minutes (20)\n* :unlock_token_expire_minutes (5)\n* :use_binary_id (false) - Use binary ids.\n* :user_active_field - Include the user active feature\n* :user_token (false) - generate tokens for channel authentication\n* :user_schema\n* :verify_user_token (fn socket, token -> Phoenix.Token.verify(socket, \"user socket\", token, max_age: 2 * 7 * 24 * 60 * 60) end\n  can also be a 3 element tuple as described above for :token_generator\n* :web_module - the name of the project's web module (`web_module: MyProjectWeb`)","ref":"Coherence.Config.html","title":"Coherence.Config","type":"module"},{"doc":"alias Coherence.Config\n\n    Config.module","ref":"Coherence.Config.html#module-examples","title":"Examples - Coherence.Config","type":"module"},{"doc":"","ref":"Coherence.Config.html#allow_silent_password_recovery_for_unknown_user/1","title":"Coherence.Config.allow_silent_password_recovery_for_unknown_user/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#allow_unconfirmed_access_for/1","title":"Coherence.Config.allow_unconfirmed_access_for/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#assigns_key/1","title":"Coherence.Config.assigns_key/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#async_rememberable?/1","title":"Coherence.Config.async_rememberable?/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#auth_module/1","title":"Coherence.Config.auth_module/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#changeset/1","title":"Coherence.Config.changeset/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#confirm_email_updates/1","title":"Coherence.Config.confirm_email_updates/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#confirmation_token_expire_days/1","title":"Coherence.Config.confirmation_token_expire_days/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#create_login/1","title":"Coherence.Config.create_login/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#credential_store/1","title":"Coherence.Config.credential_store/1","type":"function"},{"doc":"Get the configured routes.\n\nIf config[:default_routes] is nil, return the default routes, otherwise, return\nthe the configured map.","ref":"Coherence.Config.html#default_routes/0","title":"Coherence.Config.default_routes/0","type":"function"},{"doc":"iex Application.put_env(:coherence, :default_routes, %{\n      passwords: \"/passwords\",\n      sessions: \"/sessions\",\n    })\n    :ok\n    iex Coherence.Config.default_routes()\n    %{\n      passwords: \"/passwords\",\n      sessions: \"/sessions\",\n    })","ref":"Coherence.Config.html#default_routes/0-examples","title":"Examples - Coherence.Config.default_routes/0","type":"function"},{"doc":"","ref":"Coherence.Config.html#delete_login/1","title":"Coherence.Config.delete_login/1","type":"function"},{"doc":"Get the email_from configuration","ref":"Coherence.Config.html#email_from/0","title":"Coherence.Config.email_from/0","type":"function"},{"doc":"","ref":"Coherence.Config.html#email_from_email/1","title":"Coherence.Config.email_from_email/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#email_from_name/1","title":"Coherence.Config.email_from_name/1","type":"function"},{"doc":"Get the email_reply_to value.\n\nFetches `:email_reply_to` from coherence configuration\n\n  * if nil, returns {config[:email_reply_to_name], config[:email_reply_to_email]}\n  * if true, returns true\n  * if email, returns email with a deprecation logged","ref":"Coherence.Config.html#email_reply_to/0","title":"Coherence.Config.email_reply_to/0","type":"function"},{"doc":"","ref":"Coherence.Config.html#email_reply_to_email/1","title":"Coherence.Config.email_reply_to_email/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#email_reply_to_name/1","title":"Coherence.Config.email_reply_to_name/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#forwarded_invitation_fields/1","title":"Coherence.Config.forwarded_invitation_fields/1","type":"function"},{"doc":"Get a configuration item.","ref":"Coherence.Config.html#get/2","title":"Coherence.Config.get/2","type":"function"},{"doc":"Test if an option is configured and accepts a specific action","ref":"Coherence.Config.html#has_action?/2","title":"Coherence.Config.has_action?/2","type":"function"},{"doc":"Test if an options is configured.","ref":"Coherence.Config.html#has_option/1","title":"Coherence.Config.has_option/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#invitation_permitted_attributes/1","title":"Coherence.Config.invitation_permitted_attributes/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#layout/1","title":"Coherence.Config.layout/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#log_emails/1","title":"Coherence.Config.log_emails/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#logged_in_url/1","title":"Coherence.Config.logged_in_url/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#logged_out_url/1","title":"Coherence.Config.logged_out_url/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#login_cookie/1","title":"Coherence.Config.login_cookie/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#login_field/1","title":"Coherence.Config.login_field/1","type":"function"},{"doc":"Get the configured mailer adapter","ref":"Coherence.Config.html#mailer?/0","title":"Coherence.Config.mailer?/0","type":"function"},{"doc":"","ref":"Coherence.Config.html#max_failed_login_attempts/1","title":"Coherence.Config.max_failed_login_attempts/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#messages_backend/1","title":"Coherence.Config.messages_backend/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#minimum_password_length/1","title":"Coherence.Config.minimum_password_length/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#module/1","title":"Coherence.Config.module/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#opts/1","title":"Coherence.Config.opts/1","type":"function"},{"doc":"Macro to fetch the password_hash field.\n\nUse a macro here to optimize performance.","ref":"Coherence.Config.html#password_hash/0","title":"Coherence.Config.password_hash/0","type":"macro"},{"doc":"","ref":"Coherence.Config.html#password_hash_field/1","title":"Coherence.Config.password_hash_field/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#password_hashing_alg/1","title":"Coherence.Config.password_hashing_alg/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#password_reset_permitted_attributes/1","title":"Coherence.Config.password_reset_permitted_attributes/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#registration_permitted_attributes/1","title":"Coherence.Config.registration_permitted_attributes/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#rememberable_cookie_expire_hours/1","title":"Coherence.Config.rememberable_cookie_expire_hours/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#repo/1","title":"Coherence.Config.repo/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#require_current_password/1","title":"Coherence.Config.require_current_password/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#reset_token_expire_days/1","title":"Coherence.Config.reset_token_expire_days/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#router/1","title":"Coherence.Config.router/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#schema_key/1","title":"Coherence.Config.schema_key/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#session_key/1","title":"Coherence.Config.session_key/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#session_permitted_attributes/1","title":"Coherence.Config.session_permitted_attributes/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#site_name/1","title":"Coherence.Config.site_name/1","type":"function"},{"doc":"Get title","ref":"Coherence.Config.html#title/0","title":"Coherence.Config.title/0","type":"function"},{"doc":"","ref":"Coherence.Config.html#token_assigns_key/1","title":"Coherence.Config.token_assigns_key/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#token_generator/1","title":"Coherence.Config.token_generator/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#token_max_age/1","title":"Coherence.Config.token_max_age/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#token_salt/1","title":"Coherence.Config.token_salt/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#unlock_timeout_minutes/1","title":"Coherence.Config.unlock_timeout_minutes/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#unlock_token_expire_minutes/1","title":"Coherence.Config.unlock_token_expire_minutes/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#update_login/1","title":"Coherence.Config.update_login/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#use_binary_id/1","title":"Coherence.Config.use_binary_id/1","type":"function"},{"doc":"Test if Phoenix is configured to use binary ids by default","ref":"Coherence.Config.html#use_binary_id?/0","title":"Coherence.Config.use_binary_id?/0","type":"function"},{"doc":"","ref":"Coherence.Config.html#user_active_field/1","title":"Coherence.Config.user_active_field/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#user_schema/1","title":"Coherence.Config.user_schema/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#user_token/1","title":"Coherence.Config.user_token/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#verify_user_token/1","title":"Coherence.Config.verify_user_token/1","type":"function"},{"doc":"","ref":"Coherence.Config.html#web_module/1","title":"Coherence.Config.web_module/1","type":"function"},{"doc":"Confirmable allows users to confirm a new account.\n\nWhen enabled, newly created accounts are emailed send a confirmation\nemail with a confirmation link. Clicking on the confirmation link enables\nthe account.\n\nAccess to the account is disabled until the account is enabled, unless the\nthe `allow_unconfirmed_access_for` option is configured. If the account is\nnot confirmed before the `confirmation_token_expire_days` configuration days\nexpires, a new confirmation must be sent.\n\nConfirmable adds the following database columns to the user model:\n\n* :confirmation_token - a unique token required to confirm the account\n* :confirmed_at - time and date the account was confirmed\n* :confirmation_sent_at - the time and date the confirmation token was created\n\nThe following configuration is used to customize confirmable behavior:\n\n* :confirmation_token_expire_days  - number days to allow confirmation. default 5 days\n* :allow_unconfirmed_access_for - number of days to allow login access to the account before confirmation.\ndefault 0 (disabled)","ref":"Coherence.ConfirmableService.html","title":"Coherence.ConfirmableService","type":"module"},{"doc":"Confirm a user account.\n\nAdds the `:confirmed_at` datetime field on the user model.\n\ndeprecated! Please use Coherence.ControllerHelpers.unlock!/1.","ref":"Coherence.ConfirmableService.html#confirm/1","title":"Coherence.ConfirmableService.confirm/1","type":"function"},{"doc":"Confirm a user account.\n\nAdds the `:confirmed_at` datetime field on the user model.\n\ndeprecated! Please use Coherence.ControllerHelpers.unlock!/1.","ref":"Coherence.ConfirmableService.html#confirm!/1","title":"Coherence.ConfirmableService.confirm!/1","type":"function"},{"doc":"Checks if the user has been confirmed.\n\nReturns true if confirmed, false otherwise","ref":"Coherence.ConfirmableService.html#confirmed?/1","title":"Coherence.ConfirmableService.confirmed?/1","type":"function"},{"doc":"Checks if the confirmation token has expired.\n\nReturns true when the confirmation has expired.","ref":"Coherence.ConfirmableService.html#expired?/1","title":"Coherence.ConfirmableService.expired?/1","type":"function"},{"doc":"Checks if the user can access the account before confirmation.\n\nReturns true if the unconfirmed access has not expired.","ref":"Coherence.ConfirmableService.html#unconfirmed_access?/1","title":"Coherence.ConfirmableService.unconfirmed_access?/1","type":"function"},{"doc":"Handle confirmation actions.\n\nA single action, `edit`, is required for the confirmation module.","ref":"Coherence.ConfirmationController.html","title":"Coherence.ConfirmationController","type":"module"},{"doc":"Create a new confirmation token and resend the email.","ref":"Coherence.ConfirmationController.html#create/2","title":"Coherence.ConfirmationController.create/2","type":"function"},{"doc":"Handle the user's click on the confirm link in the confirmation email.\n\nValidate that the confirmation token has not expired and sets `confirmation_sent_at`\nfield to nil, marking the user as confirmed.","ref":"Coherence.ConfirmationController.html#edit/2","title":"Coherence.ConfirmationController.edit/2","type":"function"},{"doc":"Handle resending a confirmation email.\n\nRequest the user's email, reset the confirmation token and resend the email.","ref":"Coherence.ConfirmationController.html#new/2","title":"Coherence.ConfirmationController.new/2","type":"function"},{"doc":"","ref":"Coherence.ConfirmationController.html#schema/1","title":"Coherence.ConfirmationController.schema/1","type":"function"},{"doc":"Handle confirmation actions.\n\nA single action, `edit`, is required for the confirmation module.","ref":"Coherence.ConfirmationControllerBase.html","title":"Coherence.ConfirmationControllerBase","type":"module"},{"doc":"Common helper functions for Coherence Controllers.","ref":"Coherence.Controller.html","title":"Coherence.Controller","type":"module"},{"doc":"","ref":"Coherence.Controller.html#changeset/4","title":"Coherence.Controller.changeset/4","type":"function"},{"doc":"Confirm a user account.\n\nAdds the `:confirmed_at` datetime field on the user model and updates the database","ref":"Coherence.Controller.html#confirm!/1","title":"Coherence.Controller.confirm!/1","type":"function"},{"doc":"Deactivate a user.\n\nRemoves all logged in sessions for a user.","ref":"Coherence.Controller.html#deactivate_user/1","title":"Coherence.Controller.deactivate_user/1","type":"function"},{"doc":"Test if a datetime has expired.\n\nConvert the datetime from NaiveDateTime format to Timex format to do\nthe comparison given the time during in opts.","ref":"Coherence.Controller.html#expired?/2","title":"Coherence.Controller.expired?/2","type":"function"},{"doc":"expired?(user.expire_at, days: 5)\n    expired?(user.expire_at, minutes: 10)\n\n    iex> NaiveDateTime.utc_now()\n    ...> |> Coherence.Controller.expired?(days: 1)\n    false\n\n    iex> NaiveDateTime.utc_now()\n    ...> |> Coherence.Controller.shift(days: -2)\n    ...> |> Coherence.Controller.expired?(days: 1)\n    true","ref":"Coherence.Controller.html#expired?/2-examples","title":"Examples - Coherence.Controller.expired?/2","type":"function"},{"doc":"Put LayoutView\n\nAdds Config.layout if set.","ref":"Coherence.Controller.html#layout_view/2","title":"Coherence.Controller.layout_view/2","type":"function"},{"doc":"Lock a use account.\n\nSets the `:locked_at` field on the user model to the current date and time unless\nprovided a value for the optional parameter.\n\nYou can provide a date in the future to override the configured lock expiry time. You\ncan set this data far in the future to do a pseudo permanent lock.","ref":"Coherence.Controller.html#lock!/2","title":"Coherence.Controller.lock!/2","type":"function"},{"doc":"Log an error message when lockable update fails.","ref":"Coherence.Controller.html#lockable_failure/1","title":"Coherence.Controller.lockable_failure/1","type":"function"},{"doc":"Get the configured logged_in_url.","ref":"Coherence.Controller.html#logged_in_url/1","title":"Coherence.Controller.logged_in_url/1","type":"function"},{"doc":"Get the configured logged_out_url.","ref":"Coherence.Controller.html#logged_out_url/1","title":"Coherence.Controller.logged_out_url/1","type":"function"},{"doc":"Login a user.\n\nLogs in a user and redirects them to the session_create page.","ref":"Coherence.Controller.html#login_user/3","title":"Coherence.Controller.login_user/3","type":"function"},{"doc":"Logout a user.\n\nLogs out a user and redirects them to the session_delete page.","ref":"Coherence.Controller.html#logout_user/2","title":"Coherence.Controller.logout_user/2","type":"function"},{"doc":"leaves in params only permitted request parameters.\n\nSame as Rails permit, prevents mass assignment attacks.","ref":"Coherence.Controller.html#permit/2","title":"Coherence.Controller.permit/2","type":"function"},{"doc":"Get a random string of given length.\n\nReturns a random url safe encoded64 string of the given length.\nUsed to generate tokens for the various modules that require unique tokens.","ref":"Coherence.Controller.html#random_string/1","title":"Coherence.Controller.random_string/1","type":"function"},{"doc":"Plug to redirect already logged in users.","ref":"Coherence.Controller.html#redirect_logged_in/2","title":"Coherence.Controller.redirect_logged_in/2","type":"function"},{"doc":"","ref":"Coherence.Controller.html#redirect_to/3","title":"Coherence.Controller.redirect_to/3","type":"function"},{"doc":"","ref":"Coherence.Controller.html#redirect_to/4","title":"Coherence.Controller.redirect_to/4","type":"function"},{"doc":"","ref":"Coherence.Controller.html#respond_with/3","title":"Coherence.Controller.respond_with/3","type":"function"},{"doc":"Get the Router.Helpers module for the project..\n\nReturns the projects Router.Helpers module.","ref":"Coherence.Controller.html#router_helpers/0","title":"Coherence.Controller.router_helpers/0","type":"function"},{"doc":"","ref":"Coherence.Controller.html#schema_module/1","title":"Coherence.Controller.schema_module/1","type":"function"},{"doc":"Send confirmation email with token.\n\nIf the user supports confirmable, generate a token and send the email.","ref":"Coherence.Controller.html#send_confirmation/3","title":"Coherence.Controller.send_confirmation/3","type":"function"},{"doc":"Sends an email if the mailer is properly configured and add the appropriate flash.","ref":"Coherence.Controller.html#send_email_if_mailer/3","title":"Coherence.Controller.send_email_if_mailer/3","type":"function"},{"doc":"Send a user email.\n\nSends a user email given the module, model, and url. Logs the email for\ndebug purposes.\n\nNote: This function uses an apply to avoid compile warnings if the\nmailer is not selected as an option.","ref":"Coherence.Controller.html#send_user_email/3","title":"Coherence.Controller.send_user_email/3","type":"function"},{"doc":"Set view plug","ref":"Coherence.Controller.html#set_view/2","title":"Coherence.Controller.set_view/2","type":"function"},{"doc":"Shift a NaiveDateTime.","ref":"Coherence.Controller.html#shift/2","title":"Coherence.Controller.shift/2","type":"function"},{"doc":"iex> ~N(2016-10-10 10:10:10)\n    ...> |> Coherence.Controller.shift(days: -2)\n    ...> |> to_string\n    \"2016-10-08 10:10:10Z\"","ref":"Coherence.Controller.html#shift/2-examples","title":"Examples - Coherence.Controller.shift/2","type":"function"},{"doc":"Unlock a user account.\n\nClears the `:locked_at` field on the user model and updates the database.","ref":"Coherence.Controller.html#unlock!/1","title":"Coherence.Controller.unlock!/1","type":"function"},{"doc":"","ref":"Coherence.Controller.html#t:changeset/0","title":"Coherence.Controller.changeset/0","type":"type"},{"doc":"","ref":"Coherence.Controller.html#t:conn/0","title":"Coherence.Controller.conn/0","type":"type"},{"doc":"","ref":"Coherence.Controller.html#t:params/0","title":"Coherence.Controller.params/0","type":"type"},{"doc":"","ref":"Coherence.Controller.html#t:schema/0","title":"Coherence.Controller.schema/0","type":"type"},{"doc":"","ref":"Coherence.Controller.html#t:schema_or_error/0","title":"Coherence.Controller.schema_or_error/0","type":"type"},{"doc":"","ref":"Coherence.CredentialStore.Server.html","title":"Coherence.CredentialStore.Server","type":"module"},{"doc":"","ref":"Coherence.CredentialStore.Server.html#delete_credentials/1","title":"Coherence.CredentialStore.Server.delete_credentials/1","type":"function"},{"doc":"","ref":"Coherence.CredentialStore.Server.html#delete_user_logins/1","title":"Coherence.CredentialStore.Server.delete_user_logins/1","type":"function"},{"doc":"","ref":"Coherence.CredentialStore.Server.html#get_user_data/1","title":"Coherence.CredentialStore.Server.get_user_data/1","type":"function"},{"doc":"","ref":"Coherence.CredentialStore.Server.html#put_credentials/2","title":"Coherence.CredentialStore.Server.put_credentials/2","type":"function"},{"doc":"","ref":"Coherence.CredentialStore.Server.html#start_link/1","title":"Coherence.CredentialStore.Server.start_link/1","type":"function"},{"doc":"","ref":"Coherence.CredentialStore.Server.html#stop/0","title":"Coherence.CredentialStore.Server.stop/0","type":"function"},{"doc":"","ref":"Coherence.CredentialStore.Server.html#update_user_logins/1","title":"Coherence.CredentialStore.Server.update_user_logins/1","type":"function"},{"doc":"Stores current credential information.\n\nUses a Server to save logged in credentials.\n\nNote: If you restart the phoenix server, this information\nis lost, requiring the user to log in again.\n\nIf you would like to preserve login status across server restart, you\ncan enable the Rememberable option, or configure the Database\ncache on the Session plug.","ref":"Coherence.CredentialStore.Session.html","title":"Coherence.CredentialStore.Session","type":"module"},{"doc":"Deletes `credentials` from the store.\n\nReturns the current value of `credentials`, if `credentials` exists.","ref":"Coherence.CredentialStore.Session.html#delete_credentials/1","title":"Coherence.CredentialStore.Session.delete_credentials/1","type":"function"},{"doc":"Deletes the sessions for all logged in users.","ref":"Coherence.CredentialStore.Session.html#delete_user_logins/1","title":"Coherence.CredentialStore.Session.delete_user_logins/1","type":"function"},{"doc":"Gets the user data for the given credentials","ref":"Coherence.CredentialStore.Session.html#get_user_data/1","title":"Coherence.CredentialStore.Session.get_user_data/1","type":"function"},{"doc":"Puts the `user_data` for the given `credentials`.","ref":"Coherence.CredentialStore.Session.html#put_credentials/1","title":"Coherence.CredentialStore.Session.put_credentials/1","type":"function"},{"doc":"Starts a new credentials store.","ref":"Coherence.CredentialStore.Session.html#start_link/1","title":"Coherence.CredentialStore.Session.start_link/1","type":"function"},{"doc":"","ref":"Coherence.CredentialStore.Session.html#t:t/0","title":"Coherence.CredentialStore.Session.t/0","type":"type"},{"doc":"","ref":"Coherence.CredentialStore.Types.html","title":"Coherence.CredentialStore.Types","type":"module"},{"doc":"","ref":"Coherence.CredentialStore.Types.html#t:credentials/0","title":"Coherence.CredentialStore.Types.credentials/0","type":"type"},{"doc":"","ref":"Coherence.CredentialStore.Types.html#t:user_data/0","title":"Coherence.CredentialStore.Types.user_data/0","type":"type"},{"doc":"","ref":"Coherence.CredentialStore.Types.html#t:user_id/0","title":"Coherence.CredentialStore.Types.user_id/0","type":"type"},{"doc":"Database persistence of current_user data.\n\nImplement this protocol to add database storage, allowing session\ndata to survive application restarts.","ref":"Coherence.DbStore.html","title":"Coherence.DbStore","type":"protocol"},{"doc":"Delete current user credentials.","ref":"Coherence.DbStore.html#delete_credentials/2","title":"Coherence.DbStore.delete_credentials/2","type":"function"},{"doc":"Delete all logged in users.","ref":"Coherence.DbStore.html#delete_user_logins/1","title":"Coherence.DbStore.delete_user_logins/1","type":"function"},{"doc":"Get authenticated user data.","ref":"Coherence.DbStore.html#get_user_data/3","title":"Coherence.DbStore.get_user_data/3","type":"function"},{"doc":"Save authenticated user data in the database.","ref":"Coherence.DbStore.html#put_credentials/3","title":"Coherence.DbStore.put_credentials/3","type":"function"},{"doc":"","ref":"Coherence.DbStore.html#t:schema/0","title":"Coherence.DbStore.schema/0","type":"type"},{"doc":"","ref":"Coherence.DbStore.html#t:t/0","title":"Coherence.DbStore.t/0","type":"type"},{"doc":"Handle invitation actions.\n\nHandle the following actions:\n\n* new - render the send invitation form.\n* create - generate and send the invitation token.\n* edit - render the form after user clicks the invitation email link.\n* create_user - create a new user database record\n* resend - resend an invitation token email","ref":"Coherence.InvitationController.html","title":"Coherence.InvitationController","type":"module"},{"doc":"Generate and send an invitation token.\n\nCreates a new invitation token, save it to the database and send\nthe invitation email.","ref":"Coherence.InvitationController.html#create/2","title":"Coherence.InvitationController.create/2","type":"function"},{"doc":"Create a new user action.\n\nCreate a new user based from an invite token.","ref":"Coherence.InvitationController.html#create_user/2","title":"Coherence.InvitationController.create_user/2","type":"function"},{"doc":"","ref":"Coherence.InvitationController.html#do_insert/5","title":"Coherence.InvitationController.do_insert/5","type":"function"},{"doc":"Render the create user template.\n\nSets the name and email address in the form based on what was entered\nwhen the invitation was sent.","ref":"Coherence.InvitationController.html#edit/2","title":"Coherence.InvitationController.edit/2","type":"function"},{"doc":"Render the new invitation form.","ref":"Coherence.InvitationController.html#new/2","title":"Coherence.InvitationController.new/2","type":"function"},{"doc":"Resent an invitation\n\nResent the invitation based on the invitation's id.","ref":"Coherence.InvitationController.html#resend/2","title":"Coherence.InvitationController.resend/2","type":"function"},{"doc":"","ref":"Coherence.InvitationController.html#schema/1","title":"Coherence.InvitationController.schema/1","type":"function"},{"doc":"","ref":"Coherence.InvitationController.html#t:conn/0","title":"Coherence.InvitationController.conn/0","type":"type"},{"doc":"","ref":"Coherence.InvitationController.html#t:params/0","title":"Coherence.InvitationController.params/0","type":"type"},{"doc":"","ref":"Coherence.InvitationController.html#t:schema/0","title":"Coherence.InvitationController.schema/0","type":"type"},{"doc":"Handle invitation actions.\n\nHandle the following actions:\n\n* new - render the send invitation form.\n* create - generate and send the invitation token.\n* edit - render the form after user clicks the invitation email link.\n* create_user - create a new user database record\n* resend - resend an invitation token email","ref":"Coherence.InvitationControllerBase.html","title":"Coherence.InvitationControllerBase","type":"module"},{"doc":"Lockable disables an account after too many failed login attempts.\n\nEnabled with the `--lockable` installation option, after 5 failed login\nattempts, the user is locked out of their account for 5 minutes.\n\nThis option adds the following fields to the user schema:\n\n* :failed_attempts, :integer - The number of failed login attempts.\n* :locked_at, :datetime - The time and date when the account was locked.\n\nThe following configuration is used to customize lockable behavior:\n\n* :unlock_timeout_minutes (20) - The number of minutes to wait before unlocking the account.\n* :max_failed_login_attempts (5) - The number of failed login attempts before locking the account.\n\nBy default, a locked account will be unlocked after the `:unlock_timeout_minutes` expires or the\nis unlocked using the `unlock` API.\n\nIn addition, the `--unlock-with-token` option can be given to the installer to allow\na user to unlock their own account by requesting an email be sent with an link containing an\nunlock token.\n\nWith this option installed, the following field is added to the user schema:\n\n* :unlock_token, :string","ref":"Coherence.LockableService.html","title":"Coherence.LockableService","type":"module"},{"doc":"","ref":"Coherence.LockableService.html#unlock_token/1","title":"Coherence.LockableService.unlock_token/1","type":"function"},{"doc":"Interface for handling localization of build in Coherence messages.\n\nThe following module defines the behaviour for rendering internal\ncoherence messages.\n\nThe coherence mix tasks generate a messages file in the user's app\nthat uses this behaviour to ensure the user has implement all the\nrequired messages.","ref":"Coherence.Messages.html","title":"Coherence.Messages","type":"behaviour"},{"doc":"","ref":"Coherence.Messages.html#c:account_already_confirmed/0","title":"Coherence.Messages.account_already_confirmed/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:account_created_successfully/0","title":"Coherence.Messages.account_created_successfully/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:account_is_inactive/0","title":"Coherence.Messages.account_is_inactive/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:account_is_not_locked/0","title":"Coherence.Messages.account_is_not_locked/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:account_updated_successfully/0","title":"Coherence.Messages.account_updated_successfully/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:already_confirmed/0","title":"Coherence.Messages.already_confirmed/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:already_logged_in/0","title":"Coherence.Messages.already_logged_in/0","type":"callback"},{"doc":"Returns the Messages module from the users app's configuration","ref":"Coherence.Messages.html#backend/0","title":"Coherence.Messages.backend/0","type":"function"},{"doc":"","ref":"Coherence.Messages.html#c:cant_be_blank/0","title":"Coherence.Messages.cant_be_blank/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:cant_find_that_token/0","title":"Coherence.Messages.cant_find_that_token/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:confirmation_token_expired/0","title":"Coherence.Messages.confirmation_token_expired/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:could_not_find_that_email_address/0","title":"Coherence.Messages.could_not_find_that_email_address/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:forgot_your_password/0","title":"Coherence.Messages.forgot_your_password/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:http_authentication_required/0","title":"Coherence.Messages.http_authentication_required/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:incorrect_login_or_password/1","title":"Coherence.Messages.incorrect_login_or_password/1","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:invalid_confirmation_token/0","title":"Coherence.Messages.invalid_confirmation_token/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:invalid_current_password/0","title":"Coherence.Messages.invalid_current_password/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:invalid_email_or_password/0","title":"Coherence.Messages.invalid_email_or_password/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:invalid_invitation/0","title":"Coherence.Messages.invalid_invitation/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:invalid_invitation_token/0","title":"Coherence.Messages.invalid_invitation_token/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:invalid_request/0","title":"Coherence.Messages.invalid_request/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:invalid_reset_token/0","title":"Coherence.Messages.invalid_reset_token/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:invalid_unlock_token/0","title":"Coherence.Messages.invalid_unlock_token/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:invitation_already_sent/0","title":"Coherence.Messages.invitation_already_sent/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:invitation_sent/0","title":"Coherence.Messages.invitation_sent/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:invite_someone/0","title":"Coherence.Messages.invite_someone/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:mailer_required/0","title":"Coherence.Messages.mailer_required/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:maximum_login_attempts_exceeded/0","title":"Coherence.Messages.maximum_login_attempts_exceeded/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:need_an_account/0","title":"Coherence.Messages.need_an_account/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:not_locked/0","title":"Coherence.Messages.not_locked/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:password_reset_token_expired/0","title":"Coherence.Messages.password_reset_token_expired/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:problem_confirming_user_account/0","title":"Coherence.Messages.problem_confirming_user_account/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:registration_created_successfully/0","title":"Coherence.Messages.registration_created_successfully/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:required/0","title":"Coherence.Messages.required/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:resend_confirmation_email/0","title":"Coherence.Messages.resend_confirmation_email/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:reset_email_sent/0","title":"Coherence.Messages.reset_email_sent/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:restricted_area/0","title":"Coherence.Messages.restricted_area/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:send_an_unlock_email/0","title":"Coherence.Messages.send_an_unlock_email/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:sign_in/0","title":"Coherence.Messages.sign_in/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:sign_out/0","title":"Coherence.Messages.sign_out/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:signed_in_successfully/0","title":"Coherence.Messages.signed_in_successfully/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:too_many_failed_login_attempts/0","title":"Coherence.Messages.too_many_failed_login_attempts/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:unauthorized_ip_address/0","title":"Coherence.Messages.unauthorized_ip_address/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:unlock_instructions_sent/0","title":"Coherence.Messages.unlock_instructions_sent/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:user_account_confirmed_successfully/0","title":"Coherence.Messages.user_account_confirmed_successfully/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:user_already_has_an_account/0","title":"Coherence.Messages.user_already_has_an_account/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:verify_user_token/1","title":"Coherence.Messages.verify_user_token/1","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:you_are_using_an_invalid_security_token/0","title":"Coherence.Messages.you_are_using_an_invalid_security_token/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:you_must_confirm_your_account/0","title":"Coherence.Messages.you_must_confirm_your_account/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:your_account_has_been_unlocked/0","title":"Coherence.Messages.your_account_has_been_unlocked/0","type":"callback"},{"doc":"","ref":"Coherence.Messages.html#c:your_account_is_not_locked/0","title":"Coherence.Messages.your_account_is_not_locked/0","type":"callback"},{"doc":"Handle password recovery actions.\n\nController that handles the recover password feature.\n\nActions:\n\n* new - render the recover password form\n* create - verify user's email address, generate a token, and send the email\n* edit - render the reset password form\n* update - verify password, password confirmation, and update the database","ref":"Coherence.PasswordController.html","title":"Coherence.PasswordController","type":"module"},{"doc":"","ref":"Coherence.PasswordController.html#clear_password_params/1","title":"Coherence.PasswordController.clear_password_params/1","type":"function"},{"doc":"Create the recovery token and send the email","ref":"Coherence.PasswordController.html#create/2","title":"Coherence.PasswordController.create/2","type":"function"},{"doc":"Render the password and password confirmation form.","ref":"Coherence.PasswordController.html#edit/2","title":"Coherence.PasswordController.edit/2","type":"function"},{"doc":"Render the recover password form.","ref":"Coherence.PasswordController.html#new/2","title":"Coherence.PasswordController.new/2","type":"function"},{"doc":"","ref":"Coherence.PasswordController.html#recover_password/4","title":"Coherence.PasswordController.recover_password/4","type":"function"},{"doc":"","ref":"Coherence.PasswordController.html#schema/1","title":"Coherence.PasswordController.schema/1","type":"function"},{"doc":"Verify the passwords and update the database","ref":"Coherence.PasswordController.html#update/2","title":"Coherence.PasswordController.update/2","type":"function"},{"doc":"","ref":"Coherence.PasswordController.html#t:conn/0","title":"Coherence.PasswordController.conn/0","type":"type"},{"doc":"","ref":"Coherence.PasswordController.html#t:params/0","title":"Coherence.PasswordController.params/0","type":"type"},{"doc":"","ref":"Coherence.PasswordController.html#t:schema/0","title":"Coherence.PasswordController.schema/0","type":"type"},{"doc":"Handle password recovery actions.\n\nController that handles the recover password feature.\n\nActions:\n\n* new - render the recover password form\n* create - verify user's email address, generate a token, and send the email\n* edit - render the reset password form\n* update - verify password, password confirmation, and update the database","ref":"Coherence.PasswordControllerBase.html","title":"Coherence.PasswordControllerBase","type":"module"},{"doc":"This service handles reseting of passwords.\n\nInstalled with the `--recoverable` installation option, this service handles\nthe creation of the `reset_password_token`. With this installation option, the\nfollowing fields are added to the user's schema:\n\n* :reset_password_token - A random string token generated and sent to the user\n* :reset_password_sent_at - the date and time the token was created\n\nThe following configuration can be used to customize the behavior of the\nrecoverable option:\n\n* :reset_token_expire_days (2) - the expiry time of the reset token in days.","ref":"Coherence.PasswordService.html","title":"Coherence.PasswordService","type":"module"},{"doc":"Create and save a reset password token.\n\nCreates a random password reset token and saves the token in the\nuser schema along with setting the `reset_password_sent_at` to the\ncurrent time and date.","ref":"Coherence.PasswordService.html#reset_password_token/1","title":"Coherence.PasswordService.reset_password_token/1","type":"function"},{"doc":"Handle account registration actions.\n\nActions:\n\n* new - render the register form\n* create - create a new user account\n* edit - edit the user account\n* update - update the user account\n* delete - delete the user account","ref":"Coherence.RegistrationController.html","title":"Coherence.RegistrationController","type":"module"},{"doc":"Create the new user account.\n\nCreates the new user account. Create and send a confirmation if\nthis option is enabled.","ref":"Coherence.RegistrationController.html#create/2","title":"Coherence.RegistrationController.create/2","type":"function"},{"doc":"Delete a registration.","ref":"Coherence.RegistrationController.html#delete/2","title":"Coherence.RegistrationController.delete/2","type":"function"},{"doc":"Edit the registration.","ref":"Coherence.RegistrationController.html#edit/2","title":"Coherence.RegistrationController.edit/2","type":"function"},{"doc":"Render the new user form.","ref":"Coherence.RegistrationController.html#new/2","title":"Coherence.RegistrationController.new/2","type":"function"},{"doc":"","ref":"Coherence.RegistrationController.html#redirect_or_login/4","title":"Coherence.RegistrationController.redirect_or_login/4","type":"function"},{"doc":"","ref":"Coherence.RegistrationController.html#schema/1","title":"Coherence.RegistrationController.schema/1","type":"function"},{"doc":"Show the registration page.","ref":"Coherence.RegistrationController.html#show/2","title":"Coherence.RegistrationController.show/2","type":"function"},{"doc":"Update the registration.","ref":"Coherence.RegistrationController.html#update/2","title":"Coherence.RegistrationController.update/2","type":"function"},{"doc":"","ref":"Coherence.RegistrationController.html#t:conn/0","title":"Coherence.RegistrationController.conn/0","type":"type"},{"doc":"","ref":"Coherence.RegistrationController.html#t:params/0","title":"Coherence.RegistrationController.params/0","type":"type"},{"doc":"","ref":"Coherence.RegistrationController.html#t:schema/0","title":"Coherence.RegistrationController.schema/0","type":"type"},{"doc":"Handle account registration actions.\n\nActions:\n\n* new - render the register form\n* create - create a new user account\n* edit - edit the user account\n* update - update the user account\n* delete - delete the user account","ref":"Coherence.RegistrationControllerBase.html","title":"Coherence.RegistrationControllerBase","type":"module"},{"doc":"Plug to protect controllers that require login.","ref":"Coherence.RequireLogin.html","title":"Coherence.RequireLogin","type":"module"},{"doc":"","ref":"Coherence.RequireLogin.html#call/2","title":"Coherence.RequireLogin.call/2","type":"function"},{"doc":"","ref":"Coherence.RequireLogin.html#init/1","title":"Coherence.RequireLogin.init/1","type":"function"},{"doc":"","ref":"Coherence.Responders.html","title":"Coherence.Responders","type":"behaviour"},{"doc":"","ref":"Coherence.Responders.html#c:confirmation_create_error/2","title":"Coherence.Responders.confirmation_create_error/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:confirmation_create_success/2","title":"Coherence.Responders.confirmation_create_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:confirmation_update_error/2","title":"Coherence.Responders.confirmation_update_error/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:confirmation_update_success/2","title":"Coherence.Responders.confirmation_update_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:invitation_create_error/2","title":"Coherence.Responders.invitation_create_error/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:invitation_create_success/2","title":"Coherence.Responders.invitation_create_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:invitation_create_user_error/2","title":"Coherence.Responders.invitation_create_user_error/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:invitation_create_user_success/2","title":"Coherence.Responders.invitation_create_user_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:invitation_resend_error/2","title":"Coherence.Responders.invitation_resend_error/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:invitation_resend_success/2","title":"Coherence.Responders.invitation_resend_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:password_create_error/2","title":"Coherence.Responders.password_create_error/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:password_create_success/2","title":"Coherence.Responders.password_create_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:password_update_error/2","title":"Coherence.Responders.password_update_error/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:password_update_success/2","title":"Coherence.Responders.password_update_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:registration_create_error/2","title":"Coherence.Responders.registration_create_error/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:registration_create_success/2","title":"Coherence.Responders.registration_create_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:registration_delete_success/2","title":"Coherence.Responders.registration_delete_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:registration_update_error/2","title":"Coherence.Responders.registration_update_error/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:registration_update_success/2","title":"Coherence.Responders.registration_update_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:session_already_logged_in/2","title":"Coherence.Responders.session_already_logged_in/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:session_create_error/2","title":"Coherence.Responders.session_create_error/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:session_create_error_locked/2","title":"Coherence.Responders.session_create_error_locked/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:session_create_success/2","title":"Coherence.Responders.session_create_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:session_delete_success/2","title":"Coherence.Responders.session_delete_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:unlock_create_error/2","title":"Coherence.Responders.unlock_create_error/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:unlock_create_error_not_locked/2","title":"Coherence.Responders.unlock_create_error_not_locked/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:unlock_create_success/2","title":"Coherence.Responders.unlock_create_success/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:unlock_update_error/2","title":"Coherence.Responders.unlock_update_error/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:unlock_update_error_not_locked/2","title":"Coherence.Responders.unlock_update_error_not_locked/2","type":"callback"},{"doc":"","ref":"Coherence.Responders.html#c:unlock_update_success/2","title":"Coherence.Responders.unlock_update_success/2","type":"callback"},{"doc":"Handles routing for Coherence.","ref":"Coherence.Router.html","title":"Coherence.Router","type":"module"},{"doc":"Add the following to your `web/router.ex` file\n\n    defmodule MyProject.Router do\n      use MyProject.Web, :router\n      use Coherence.Router         # Add this\n\n      pipeline :browser do\n        plug :accepts, [\"html\"]\n        # ...\n        plug Coherence.Authentication.Session           # Add this\n      end\n\n      pipeline :protected do\n        plug :accepts, [\"html\"]\n        # ...\n        plug Coherence.Authentication.Session, protected: true\n      end\n\n      scope \"/\" do\n        pipe_through :browser\n        coherence_routes()\n      end\n\n      scope \"/\" do\n        pipe_through :protected\n        coherence_routes :protected\n      end\n      # ...\n    end\n\nAlternatively, you may want to use the login plug in individual controllers. In\nthis case, you can have one pipeline, one scope and call `coherence_routes :all`.\nIn this case, it will add both the public and protected routes.","ref":"Coherence.Router.html#module-usage","title":"Usage - Coherence.Router","type":"module"},{"doc":"Coherence Router macro.\n\nUse this macro to define the various Coherence Routes.","ref":"Coherence.Router.html#coherence_routes/2","title":"Coherence.Router.coherence_routes/2","type":"macro"},{"doc":"# Routes that don't require authentication\n    scope \"/\" do\n      pipe_through :browser\n      coherence_routes\n    end\n\n    # Routes that require authentication\n    scope \"/\" do\n      pipe_through :protected\n      coherence_routes :protected\n    end","ref":"Coherence.Router.html#coherence_routes/2-examples","title":"Examples: - Coherence.Router.coherence_routes/2","type":"macro"},{"doc":"Add Coherence support to a User schema module.\n\nAdd `use Coherence.Schema, opts \\ []` to your User module to add a number of\nModule functions and helpers.\n\nThe optional `opt` parameter can be used to disable options enabled in the\nglobal configuration by passing `option: false`\n\nFor example,\n\n    defmodule MyProject.User do\n      use MyProject.Web, :model\n      use Coherence.Schema, invitable: false\n\n\nThe following functions are added regardless of the options configured:\n\n* `authenticatable?/0` - Returns true if the option is configured.\n* `registerable?/0` - Returns true if the option is configured.\n* `confirmable?/0` - Returns true if the option is configured.\n* `trackable?/0` - Returns true if the option is configured.\n* `trackable_table?/0` - Returns true if the option is configured.\n* `recoverable?/0` - Returns true if the option is configured.\n* `lockable?/0` - Returns true if the option is configured.\n* `invitable?/0` - Returns true if the option is configured.\n* `unlockable_with_token?/0` - Returns true if the option is configured.\n\nThe following functions are available when `authenticatable?/0` returns true:\n\n* `checkpw/2` - Validate password.\n* `encrypt_password/1` - encrypted a password using ` .hashpwsalt`\n* `validate_coherence/2` - run the coherence password validations.\n* `validate_password/2` - Used by `validate_coherence for password validation`\n\nThe following functions are available when `confirmable?/0` returns true.\n\n* `confirmed?/1` - Has the given user been confirmed?\n* `confirm/1` - Return a changeset to confirm the given user\n\nThe following functions are available when `lockable?/0` returns true.\n\n* `locked?/1` - Is the given user locked?\n* `lock/1` - Return a changeset to lock the given user\n* `unlock/1` - Return a changeset to unlock the given user\n\nThe `coherence_schema/1` macro is used to add the configured schema fields to the User models schema.\n\nThe `coherence_fields/0` function is used to return the validation fields appropriate for the selected options.","ref":"Coherence.Schema.html","title":"Coherence.Schema","type":"module"},{"doc":"The following is an example User module when the :authenticatable is used:\n\n    defmodule MyProject.User do\n      use MyProject.Web, :model\n      use Coherence.Schema\n\n      schema \"users\" do\n        field :name, :string\n        field :email, :string\n        coherence_schema\n\n        timestamps\n      end\n\n      @required_fields ~w(name email)\n      @optional_fields ~w() ++ coherence_fields\n\n      def changeset(model, params \\ %{}) do\n        model\n        |> cast(params, @required_fields, @optional_fields)\n        |> unique_constraint(:email)\n        |> validate_coherence(params)\n      end\n\n      def changeset(model, params, :password) do\n        model\n        |> cast(params, ~w(password password_confirmation reset_password_token reset_password_sent_at))\n        |> validate_coherence_password_reset(params)\n      end\n    end","ref":"Coherence.Schema.html#module-examples","title":"Examples: - Coherence.Schema","type":"module"},{"doc":"You may have an application that has its own setting for enabling/disabling confirmable\naccounts at runtime. To enable this:\n\n    defmodule MyProject.User do\n      use MyProject.Web, :model\n      use Coherence.Schema\n      # ...\n      def confirmable? do\n        # test if confirmation is enable\n      end\n\n      def changeset(schema, params) do\n        schema\n        |> cast(params, @all_params ++ coherence_fields())\n        # ...\n        |> prepare_changes(&prepare_confirmation/1)\n      end\n\n      def prepare_confirmation(%{valid?: true} = changeset) do\n        if local_settings_require_confirmation? do\n          changeset\n        else\n          changeset\n          |> put_change(:confirmed_at, NaiveDateTime.utc_now())\n          |> put_change(:confirmation_token, nil)\n        end\n      end\n\n      def prepare_confirmation(changeset) do\n        changeset\n      end\n    end","ref":"Coherence.Schema.html#module-configurable-confirmable","title":"Configurable confirmable - Coherence.Schema","type":"module"},{"doc":"Get a list of the configured database fields.\n\nReturns a list of fields that can be appended to your @option_fields used\nin your models changeset cast.\n\nFor example, for `Coherence.Config.opts == [:authenticatable, :recoverable]`\n`coherence_fields/0` will return:\n\n    ~w(password_hash password password_confirmation reset_password_token reset_password_sent_at)a","ref":"Coherence.Schema.html#coherence_fields/0","title":"Coherence.Schema.coherence_fields/0","type":"function"},{"doc":"Add configure schema fields.\n\nAdds the schema fields to the schema block for the options selected.\nOnly the fields for configured options are added.\n\nFor example, for `Coherence.Config.opts == [:authenticatable, :recoverable]`\n`coherence_schema` used in the following context:\n\n    defmodule MyProject.User do\n      use MyProject.Web, :model\n      use Coherence.Schema\n\n      schema \"users\" do\n        field :name, :string\n        field :email, :string\n        coherence_schema\n\n        timestamps\n      end\n\nWill compile a schema to the following:\n\n    defmodule MyProject.User do\n      use MyProject.Web, :model\n      use Coherence.Schema\n\n      schema \"users\" do\n        field :name, :string\n        field :email, :string\n        field :active, :boolean, default: true\n\n        # authenticatable\n        field :password_hash, :string\n        field :password, :string, virtual: true\n        field :password_confirmation, :string, virtual: true\n\n        # recoverable\n        field :reset_password_token, :string\n        field :reset_password_sent_at, NaiveDateTime\n\n        timestamps\n      end","ref":"Coherence.Schema.html#coherence_schema/0","title":"Coherence.Schema.coherence_schema/0","type":"macro"},{"doc":"Default permitted attributes used when they are not configured.\n\nDefaults to be used when the entry is not found in configuration. These are\nmainly used when a pre 0.5.1 project is updated.","ref":"Coherence.Schema.html#permitted_attributes_default/1","title":"Coherence.Schema.permitted_attributes_default/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html","title":"Coherence.Schemas","type":"module"},{"doc":"","ref":"Coherence.Schemas.html#change_invitation/0","title":"Coherence.Schemas.change_invitation/0","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#change_invitation/1","title":"Coherence.Schemas.change_invitation/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#change_invitation/2","title":"Coherence.Schemas.change_invitation/2","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#change_rememberable/0","title":"Coherence.Schemas.change_rememberable/0","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#change_rememberable/1","title":"Coherence.Schemas.change_rememberable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#change_rememberable/2","title":"Coherence.Schemas.change_rememberable/2","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#change_trackable/0","title":"Coherence.Schemas.change_trackable/0","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#change_trackable/1","title":"Coherence.Schemas.change_trackable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#change_trackable/2","title":"Coherence.Schemas.change_trackable/2","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#change_user/0","title":"Coherence.Schemas.change_user/0","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#change_user/1","title":"Coherence.Schemas.change_user/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#change_user/2","title":"Coherence.Schemas.change_user/2","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#create/1","title":"Coherence.Schemas.create/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#create!/1","title":"Coherence.Schemas.create!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#create_invitation/1","title":"Coherence.Schemas.create_invitation/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#create_invitation!/1","title":"Coherence.Schemas.create_invitation!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#create_rememberable/1","title":"Coherence.Schemas.create_rememberable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#create_rememberable!/1","title":"Coherence.Schemas.create_rememberable!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#create_trackable/1","title":"Coherence.Schemas.create_trackable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#create_trackable!/1","title":"Coherence.Schemas.create_trackable!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#create_user/1","title":"Coherence.Schemas.create_user/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#create_user!/1","title":"Coherence.Schemas.create_user!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#delete/1","title":"Coherence.Schemas.delete/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#delete!/1","title":"Coherence.Schemas.delete!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#delete_all/1","title":"Coherence.Schemas.delete_all/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#delete_invitation/1","title":"Coherence.Schemas.delete_invitation/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#delete_invitation!/1","title":"Coherence.Schemas.delete_invitation!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#delete_rememberable/1","title":"Coherence.Schemas.delete_rememberable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#delete_rememberable!/1","title":"Coherence.Schemas.delete_rememberable!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#delete_trackable/1","title":"Coherence.Schemas.delete_trackable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#delete_trackable!/1","title":"Coherence.Schemas.delete_trackable!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_by_invitation/1","title":"Coherence.Schemas.get_by_invitation/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_by_rememberable/1","title":"Coherence.Schemas.get_by_rememberable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_by_trackable/1","title":"Coherence.Schemas.get_by_trackable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_by_user/1","title":"Coherence.Schemas.get_by_user/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_invitation/1","title":"Coherence.Schemas.get_invitation/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_invitation!/1","title":"Coherence.Schemas.get_invitation!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_rememberable/1","title":"Coherence.Schemas.get_rememberable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_rememberable!/1","title":"Coherence.Schemas.get_rememberable!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_trackable/1","title":"Coherence.Schemas.get_trackable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_trackable!/1","title":"Coherence.Schemas.get_trackable!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_user/1","title":"Coherence.Schemas.get_user/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_user!/1","title":"Coherence.Schemas.get_user!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#get_user_by_email/1","title":"Coherence.Schemas.get_user_by_email/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#last_trackable/1","title":"Coherence.Schemas.last_trackable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#list_by_invitation/1","title":"Coherence.Schemas.list_by_invitation/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#list_by_rememberable/1","title":"Coherence.Schemas.list_by_rememberable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#list_by_trackable/1","title":"Coherence.Schemas.list_by_trackable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#list_by_user/1","title":"Coherence.Schemas.list_by_user/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#list_invitation/0","title":"Coherence.Schemas.list_invitation/0","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#list_invitation/1","title":"Coherence.Schemas.list_invitation/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#list_rememberable/0","title":"Coherence.Schemas.list_rememberable/0","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#list_rememberable/1","title":"Coherence.Schemas.list_rememberable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#list_trackable/0","title":"Coherence.Schemas.list_trackable/0","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#list_trackable/1","title":"Coherence.Schemas.list_trackable/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#list_user/0","title":"Coherence.Schemas.list_user/0","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#query_by/2","title":"Coherence.Schemas.query_by/2","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#schema/1","title":"Coherence.Schemas.schema/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#update/1","title":"Coherence.Schemas.update/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#update!/1","title":"Coherence.Schemas.update!/1","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#update_invitation/2","title":"Coherence.Schemas.update_invitation/2","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#update_invitation!/2","title":"Coherence.Schemas.update_invitation!/2","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#update_rememberable/2","title":"Coherence.Schemas.update_rememberable/2","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#update_rememberable!/2","title":"Coherence.Schemas.update_rememberable!/2","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#update_trackable/2","title":"Coherence.Schemas.update_trackable/2","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#update_trackable!/2","title":"Coherence.Schemas.update_trackable!/2","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#update_user/2","title":"Coherence.Schemas.update_user/2","type":"function"},{"doc":"","ref":"Coherence.Schemas.html#update_user!/2","title":"Coherence.Schemas.update_user!/2","type":"function"},{"doc":"Handle the authentication actions.\n\nModule used for the session controller when the parent project does not\ngenerate controllers. Most of the work is done by the\n`Coherence.SessionControllerBase` inclusion.","ref":"Coherence.SessionController.html","title":"Coherence.SessionController","type":"module"},{"doc":"Check if the user is confirmed.","ref":"Coherence.SessionController.html#confirmed_access?/1","title":"Coherence.SessionController.confirmed_access?/1","type":"function"},{"doc":"Login the user.\n\nFind the user based on the login_field. Hash the given password and verify it\nmatches the value stored in the database. Login proceeds only if the following\nother conditions are satisfied:\n\n* Confirmation is enabled and the user has been confirmed.\n* Lockable is enabled and the user is not locked.\n\nIf the Trackable option is enabled, the trackable fields are update.\n\nIf the provided password is not correct, and the lockable option is enabled check\nto see if the maximum login attempts threshold is exceeded. If so, lock the account.\n\nIf the rememberable option is enabled, create a new series and rememberable token,\ncreate a new cookie and update the database.","ref":"Coherence.SessionController.html#create/2","title":"Coherence.SessionController.create/2","type":"function"},{"doc":"Logout the user.\n\nDelete the user's session, track the logout and delete the rememberable cookie.","ref":"Coherence.SessionController.html#delete/2","title":"Coherence.SessionController.delete/2","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#delete_expired_tokens!/1","title":"Coherence.SessionController.delete_expired_tokens!/1","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#do_lockable/4","title":"Coherence.SessionController.do_lockable/4","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#do_valid_login/4","title":"Coherence.SessionController.do_valid_login/4","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#failed_login/3","title":"Coherence.SessionController.failed_login/3","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#gen_cookie/3","title":"Coherence.SessionController.gen_cookie/3","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#get_invalid_login!/4","title":"Coherence.SessionController.get_invalid_login!/4","type":"function"},{"doc":"Retrieve the login cookie.","ref":"Coherence.SessionController.html#get_login_cookie/1","title":"Coherence.SessionController.get_login_cookie/1","type":"function"},{"doc":"Get the user from the database.","ref":"Coherence.SessionController.html#get_login_user/3","title":"Coherence.SessionController.get_login_user/3","type":"function"},{"doc":"Fetch a rememberable database record.","ref":"Coherence.SessionController.html#get_rememberables/1","title":"Coherence.SessionController.get_rememberables/1","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#get_valid_login!/4","title":"Coherence.SessionController.get_valid_login!/4","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#hash/1","title":"Coherence.SessionController.hash/1","type":"function"},{"doc":"Render the login form.","ref":"Coherence.SessionController.html#new/2","title":"Coherence.SessionController.new/2","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#put_flash_inactive_user/1","title":"Coherence.SessionController.put_flash_inactive_user/1","type":"function"},{"doc":"Callback for the authenticate plug.\n\nValidate the rememberable cookie. If valid, generate a new token,\nkeep the same series number. Update the rememberable database with\nthe new token. Save the new cookie.","ref":"Coherence.SessionController.html#rememberable_callback/5","title":"Coherence.SessionController.rememberable_callback/5","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#reset_failed_attempts/3","title":"Coherence.SessionController.reset_failed_attempts/3","type":"function"},{"doc":"Save the login cookie.","ref":"Coherence.SessionController.html#save_login_cookie/5","title":"Coherence.SessionController.save_login_cookie/5","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#save_rememberable/3","title":"Coherence.SessionController.save_rememberable/3","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#schema/1","title":"Coherence.SessionController.schema/1","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#track_login/4","title":"Coherence.SessionController.track_login/4","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#user_active?/1","title":"Coherence.SessionController.user_active?/1","type":"function"},{"doc":"Validate a user' login.","ref":"Coherence.SessionController.html#valid_user_login?/2","title":"Coherence.SessionController.valid_user_login?/2","type":"function"},{"doc":"Validate the login cookie.\n\nCheck the following conditions:\n\n* a record exists for the user, the series, but a different token\n  * assume a fraud case\n  * remove the rememberable cookie and delete the session\n* a record exists for the user, the series, and the token\n  * a valid remembered user\n* otherwise, this is an unknown user.","ref":"Coherence.SessionController.html#validate_login/3","title":"Coherence.SessionController.validate_login/3","type":"function"},{"doc":"","ref":"Coherence.SessionController.html#t:conn/0","title":"Coherence.SessionController.conn/0","type":"type"},{"doc":"","ref":"Coherence.SessionController.html#t:params/0","title":"Coherence.SessionController.params/0","type":"type"},{"doc":"","ref":"Coherence.SessionController.html#t:schema/0","title":"Coherence.SessionController.schema/0","type":"type"},{"doc":"Default SessionConroller action and support functions.\n\nInjects all the default handles for a session controller through a `__using__`\nmacro. The following functions are available for override:\n\n* login_cookie: 0\n* get_login_cookie: 1\n* new: 2\n* create: 2\n* confirmed_access?: 1\n* do_lockable: 4\n* delete: 2\n* reset_failed_attempts: 3\n* failed_login: 3\n* put_flash_inactive_user: 1\n* rememberable_callback: 5\n* do_rememberable_callback: 5\n* do_valid_login: 4\n* save_login_cookie: 5\n* save_rememberable: 3\n* get_rememberables: 1\n* validate_login: 3\n* get_invalid_login!: 4\n* delete_expired_tokens!: 1\n* hash: 1\n* gen_cookie: 3\n* user_active?: 1\n* schema: 1","ref":"Coherence.SessionControllerBase.html","title":"Coherence.SessionControllerBase","type":"module"},{"doc":"def MyProjectWeb.Coherence.SessionController do\n      use MyProjectWeb.Coherence, :controller\n      use Coherence.SessionControllerBase, schemas: MyProject.Coherence.Schemas\n\n      plug :layout_view, view: Coherence.SessionView, caller: __MODULE__\n      plug :redirect_logged_in when action in [:new, :create]\n    end","ref":"Coherence.SessionControllerBase.html#module-examples","title":"Examples - Coherence.SessionControllerBase","type":"module"},{"doc":"Support functions for Coherence sessions.","ref":"Coherence.SessionService.html","title":"Coherence.SessionService","type":"module"},{"doc":"Create a signed Phoenix token for a given user","ref":"Coherence.SessionService.html#sign_user_token/3","title":"Coherence.SessionService.sign_user_token/3","type":"function"},{"doc":"Verify a signed Phoenix Token.","ref":"Coherence.SessionService.html#verify_user_token/3","title":"Coherence.SessionService.verify_user_token/3","type":"function"},{"doc":"Supervisor to start Coherence services.\n\nStarts the configured credential store server. Also starts\nthe RememberableServer if this option is configured.","ref":"Coherence.Supervisor.html","title":"Coherence.Supervisor","type":"module"},{"doc":"Trackable tracks login information for each login.\n\nYou have two choices to track logins.\n\n1. Add trackable fields to your user schema.\n2. Create a separate trackable table.\n\n# User Schema Trackable Fields\n\nUse the `--trackable` installation option to enable this feature.\n\nTrackable adds the following fields to the user schema:\n\n* :sign_in_count - Increments each time a user logs in.\n* :current_sign_in_at - The time and date the user logged in.\n* :current_sign_in_ip - The IP address of the logged in user.\n* :last_sign_in_at: last_at - The previous login time and date\n* :last_sign_in_ip: last_ip - The previous login IP address\n\n# Trackable Table\n\nThis feature provides full audit capability around user logins.\n\nUse the `--trackable-table` installation option to use a separate trackable table/schema.\n\nTrackable-table creates a Trackable schema with the following fields:\n\n* :action - The action that generated the entry. Values are `:login, :logout, :password_reset, :failed_login, :lock, :unlock, :unlock_token`\n* :sign_in_count - Increments each time a user logs in.\n* :current_sign_in_at - The time and date the user logged in.\n* :current_sign_in_ip - The IP address of the logged in user.\n* :last_sign_in_at: last_at - The previous login time and date\n* :last_sign_in_ip: last_ip - The previous login IP address\n\nNote, the `--trackable` and `--trackable-table` installation options are\nmutually exclusive.","ref":"Coherence.TrackableService.html","title":"Coherence.TrackableService","type":"module"},{"doc":"","ref":"Coherence.TrackableService.html#track/3","title":"Coherence.TrackableService.track/3","type":"function"},{"doc":"","ref":"Coherence.TrackableService.html#track_failed_login/3","title":"Coherence.TrackableService.track_failed_login/3","type":"function"},{"doc":"","ref":"Coherence.TrackableService.html#track_lock/3","title":"Coherence.TrackableService.track_lock/3","type":"function"},{"doc":"Track user login details.\n\nSaves the ip address and timestamp when the user logs in.\n\nA value of true in the third argument indicates that the `:trackable option`\nis configured. A Value of true in the fourth argument indicates that the\n`:trackable_table` is configured.","ref":"Coherence.TrackableService.html#track_login/4","title":"Coherence.TrackableService.track_login/4","type":"function"},{"doc":"Track user logout.\n\nUpdates the `last_sign_in_at` and `last_sign_in_at` fields. Clears the\n`current_sign_in_at` and `current_sign_in_ip` fields.\n\nA value of true in the third argument indicates that the `:trackable option`\nis configured. A Value of true in the fourth argument indicates that the\n`:trackable_table` is configured.","ref":"Coherence.TrackableService.html#track_logout/4","title":"Coherence.TrackableService.track_logout/4","type":"function"},{"doc":"","ref":"Coherence.TrackableService.html#track_password_reset/3","title":"Coherence.TrackableService.track_password_reset/3","type":"function"},{"doc":"","ref":"Coherence.TrackableService.html#track_unlock/3","title":"Coherence.TrackableService.track_unlock/3","type":"function"},{"doc":"","ref":"Coherence.TrackableService.html#track_unlock_token/3","title":"Coherence.TrackableService.track_unlock_token/3","type":"function"},{"doc":"","ref":"Coherence.TrackableService.html#t:conn/0","title":"Coherence.TrackableService.conn/0","type":"type"},{"doc":"","ref":"Coherence.TrackableService.html#t:params/0","title":"Coherence.TrackableService.params/0","type":"type"},{"doc":"","ref":"Coherence.TrackableService.html#t:schema/0","title":"Coherence.TrackableService.schema/0","type":"type"},{"doc":"Handle unlock_with_token actions.\n\nThis controller provides the ability generate an unlock token, send\nthe user an email and unlocking the account with a valid token.\n\nBasic locking and unlocking does not use this controller.","ref":"Coherence.UnlockController.html","title":"Coherence.UnlockController","type":"module"},{"doc":"Create and send the unlock token.","ref":"Coherence.UnlockController.html#create/2","title":"Coherence.UnlockController.create/2","type":"function"},{"doc":"Handle the unlock link click.","ref":"Coherence.UnlockController.html#edit/2","title":"Coherence.UnlockController.edit/2","type":"function"},{"doc":"Render the send reset link form.","ref":"Coherence.UnlockController.html#new/2","title":"Coherence.UnlockController.new/2","type":"function"},{"doc":"","ref":"Coherence.UnlockController.html#schema/1","title":"Coherence.UnlockController.schema/1","type":"function"},{"doc":"","ref":"Coherence.UnlockController.html#t:conn/0","title":"Coherence.UnlockController.conn/0","type":"type"},{"doc":"","ref":"Coherence.UnlockController.html#t:params/0","title":"Coherence.UnlockController.params/0","type":"type"},{"doc":"","ref":"Coherence.UnlockController.html#t:schema/0","title":"Coherence.UnlockController.schema/0","type":"type"},{"doc":"Handle unlock_with_token actions.\n\nThis controller provides the ability generate an unlock token, send\nthe user an email and unlocking the account with a valid token.\n\nBasic locking and unlocking does not use this controller.","ref":"Coherence.UnlockControllerBase.html","title":"Coherence.UnlockControllerBase","type":"module"},{"doc":"Plug to validate the given option is enabled in the project's configuration.","ref":"Coherence.ValidateOption.html","title":"Coherence.ValidateOption","type":"module"},{"doc":"","ref":"Coherence.ValidateOption.html#call/2","title":"Coherence.ValidateOption.call/2","type":"function"},{"doc":"","ref":"Coherence.ValidateOption.html#init/1","title":"Coherence.ValidateOption.init/1","type":"function"},{"doc":"Coherence setting for web resources.\n\nSimilar to a project's Web module","ref":"CoherenceWeb.html","title":"CoherenceWeb","type":"module"},{"doc":"When used, dispatch to the appropriate controller/view/etc.","ref":"CoherenceWeb.html#__using__/1","title":"CoherenceWeb.__using__/1","type":"macro"},{"doc":"","ref":"CoherenceWeb.html#service/0","title":"CoherenceWeb.service/0","type":"function"},{"doc":"This task will clean most of the files installed by the `mix coh.install` task.\n\nProjects created with both `coh.install` and `coherence.install` can be\ncleaned with this task. It will auto detect the project structure and remove\nthe appropriate files.","ref":"Mix.Tasks.Coh.Clean.html","title":"mix coh.clean","type":"task"},{"doc":"# Clean all the installed files\n    mix coh.clean --all\n\n    # Clean only the installed view and template files\n    mix coh.clean --views --templates\n\n    # Clean all but the models\n    mix coh.clean --all --no-models\n\n    # Prompt once to confirm the removal\n    mix coh.clean --all --confirm-once\n\n    # Clean installed options\n    mix coh.clean --options rememberable\n    mix coh.clean --options \"registerable invitable trackable\"\n\nThe following options are supported:\n\n* `--all` -- clean all files\n* `--views` -- clean view files\n* `--templates` -- clean template files\n* `--models` -- clean models files\n* `--controllers` -- clean controller files\n* `--email` -- clean email files\n* `--web` -- clean the web/coherence_web.ex file\n* `--messages` -- clean the web/coherence_messages.ex file\n* `--migrations` -- clean the migration files\n* `--options` -- Clean one or more specific options\n* `--dry-run` -- Show what will be removed, but don't actually remove any files\n* `--confirm-once` -- confirm once before removing all selected files\n\nDisable options:\n\n* `--no-confirm` - don't confirm before removing files","ref":"Mix.Tasks.Coh.Clean.html#module-examples","title":"Examples - mix coh.clean","type":"task"},{"doc":"","ref":"Mix.Tasks.Coh.Clean.html#run/1","title":"Mix.Tasks.Coh.Clean.run/1","type":"function"},{"doc":"Generate Coherence controllers.\n\nUse this task to generate the Coherence controllers when\nyou what to customize the controllers' behavior.","ref":"Mix.Tasks.Coh.Gen.Controllers.html","title":"mix coh.gen.controllers","type":"task"},{"doc":"# Install all the controllers for the coherence opts\n    # defined in your `config/config.exs` file\n    mix coh.gen.controllers\n\nOnce the controllers have been generated, you must update your\n`router.ex` file to properly scope the generated controller(s).\n\nFor example:\n\n    # lib/my_app_web\n    def MyAppWeb.Router do\n      # ...\n\n      scope \"/\", MyAppWeb do\n        pipe_through :browser\n        coherence_routes()\n      end\n      scope \"/\", MyAppWeb do\n        pipe_through :protected\n        coherence_routes :protected\n      end\n      # ...\n    end","ref":"Mix.Tasks.Coh.Gen.Controllers.html#module-examples","title":"Examples - mix coh.gen.controllers","type":"task"},{"doc":"* `--web-path` override the web path\n* `--no-confirm` silently overwrite files","ref":"Mix.Tasks.Coh.Gen.Controllers.html#module-options","title":"Options - mix coh.gen.controllers","type":"task"},{"doc":"","ref":"Mix.Tasks.Coh.Gen.Controllers.html#copy_from/6","title":"Mix.Tasks.Coh.Gen.Controllers.copy_from/6","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Gen.Controllers.html#do_default_opts/2","title":"Mix.Tasks.Coh.Gen.Controllers.do_default_opts/2","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Gen.Controllers.html#run/1","title":"Mix.Tasks.Coh.Gen.Controllers.run/1","type":"function"},{"doc":"Configure the Coherence User Model for your Phoenix application. Coherence\nis composed of a number of modules that can be enabled with this installer.\n\nThis installer will normally do the following unless given an option not to do so:\n\n* Append the :coherence configuration to your `config/config.exs` file.\n* Generate appropriate migration files.\n* Generate appropriate view files.\n* Generate appropriate template files.\n* Generate a `web/coherence_web.ex` file.\n* Generate a `web/coherence_messages.ex` file.\n* Generate a `web/models/user.ex` file if one does not already exist.","ref":"Mix.Tasks.Coh.Install.html","title":"mix coh.install","type":"task"},{"doc":"# Install with only the `authenticatable` option\n    mix coh.install\n\n    # Install all the options except `confirmable` and `invitable`\n    mix coh.install --full\n\n    # Install all the options except `invitable`\n    mix coh.install --full-confirmable\n\n    # Install all the options except `confirmable`\n    mix coh.install --full-invitable\n\n    # Install the `full` options except `lockable` and `trackable`\n    mix coh.install --full --no-lockable --no-trackable","ref":"Mix.Tasks.Coh.Install.html#module-install-examples","title":"Install Examples - mix coh.install","type":"task"},{"doc":"# Reinstall with defaults (--silent --no-migrations --no-config --confirm-once)\n    mix coh.install --reinstall\n\n    # Confirm to overwrite files, show instructions, and generate migrations\n    mix coh.install --reinstall --no-confirm-once --with-migrations","ref":"Mix.Tasks.Coh.Install.html#module-reinstall-examples","title":"Reinstall Examples - mix coh.install","type":"task"},{"doc":"A Coherence configuration will be appended to your `config/config.exs` file unless\nthe `--no-config` option is given.\n\nA `--model=\"SomeModule tablename\"` option can be given to override the default User module.\n\nA `--repo=CustomRepo` option can be given to override the default Repo module\n\nA `--router=CustomRouter` option can be given to override the default Router module\n\nA `--web-path=\"lib/my_project/web\"` option can be given to specify the web path\n\nA `--default` option will include only `authenticatable`\n\nA `--full` option will include options `authenticatable`, `recoverable`, `lockable`, `trackable`, `unlockable_with_token`, `registerable`\n\nA `--full-confirmable` option will include the `--full` options in addition to the `--confirmable` option\n\nA `--full-invitable` option will include the `--full` options in addition to the `--invitable` option\n\nAn `--authenticatable` option provides authentication support to your User model.\n\nA `--recoverable` option provides the ability to request a password reset email.\n\nA `--lockable` option provides login locking after too many failed login attempts.\n\nAn `--unlockable-with-token` option provides the ability to request an unlock email.\n\nA `--trackable` option provides login count, current login timestamp, current login ip, last login timestamp, last login ip in your User model.\n\nA `--trackable-table` option provides `trackable` fields in the `trackables` table.\n\nA `--confirmable` option provides support for confirmation email before the account can be logged in.\n\nAn `--invitable` option provides support for invitation emails, allowing the new user to create their account including password creation.\n\nA `--registerable` option provides support for new users to register for an account\n\nA `--rememberable` option provides a remember me? check box for persistent logins\n\nA `--migration-path` option to set the migration path\n\nA `--module` option to override the module\n\nA `--web-module` option to override the web module\n\nA `--installed-options` option to list the previous install options\n\nA `--reinstall` option to reinstall the coherence boilerplate based on your existing configuration options\n\nA `--silent` option to disable printing instructions\n\nA `--confirm-once` option to only confirm overwriting existing files once\n\nA `--with-migrations` option to reinstall migrations. only valid for --reinstall option\n\nA `--layout` (false) generate layout template and view\n\nA `--user-active-field` (false) add active field to user schema and disable logins when set to false.\n\nA `--password-hashing-alg` (Comeonin.Bcrypt) add a different password hashing algorithm","ref":"Mix.Tasks.Coh.Install.html#module-option-list","title":"Option list - mix coh.install","type":"task"},{"doc":"* `--no-config` -- Don't append to your `config/config.exs` file.\n* `--no-web` -- Don't create the `coherence_web.ex` file.\n* `--no-messages` -- Don't create the `coherence_messages.ex` file.\n* `--no-views` -- Don't create the `web/views/coherence/` files.\n* `--no-migrations` -- Don't create the migration files.\n* `--no-templates` -- Don't create the `web/templates/coherence` files.\n* `--no-boilerplate` -- Don't create any of the boilerplate files.\n* `--no-models` -- Don't generate the model file.\n* `--no-confirm` -- Don't confirm overwriting files.","ref":"Mix.Tasks.Coh.Install.html#module-disable-options","title":"Disable Options - mix coh.install","type":"task"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#all_options/0","title":"Mix.Tasks.Coh.Install.all_options/0","type":"function"},{"doc":"Copies files from source dir to target dir\naccording to the given map.\nFiles are evaluated against EEx according to\nthe given binding.","ref":"Mix.Tasks.Coh.Install.html#copy_from/6","title":"Mix.Tasks.Coh.Install.copy_from/6","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#find_existing_model/2","title":"Mix.Tasks.Coh.Install.find_existing_model/2","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#gen_coherence_templates/1","title":"Mix.Tasks.Coh.Install.gen_coherence_templates/1","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#gen_coherence_views/1","title":"Mix.Tasks.Coh.Install.gen_coherence_views/1","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#gen_layout_template/1","title":"Mix.Tasks.Coh.Install.gen_layout_template/1","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#gen_schema_schema/2","title":"Mix.Tasks.Coh.Install.gen_schema_schema/2","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#gen_schemas_module/1","title":"Mix.Tasks.Coh.Install.gen_schemas_module/1","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#get_config_options/1","title":"Mix.Tasks.Coh.Install.get_config_options/1","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#get_config_options/2","title":"Mix.Tasks.Coh.Install.get_config_options/2","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#list_config_options/2","title":"Mix.Tasks.Coh.Install.list_config_options/2","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#print_installed_options/1","title":"Mix.Tasks.Coh.Install.print_installed_options/1","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#run/1","title":"Mix.Tasks.Coh.Install.run/1","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#template_files/0","title":"Mix.Tasks.Coh.Install.template_files/0","type":"function"},{"doc":"","ref":"Mix.Tasks.Coh.Install.html#view_files/0","title":"Mix.Tasks.Coh.Install.view_files/0","type":"function"},{"doc":"Define controller action redirection behaviour.\n\nDefines the default redirect functions for each of the controller\nactions that perform redirects. By using this Module you get the following\nfunctions:\n\n* session_create/2\n* session_delete/2\n* password_create/2\n* password_update/2,\n* unlock_create_not_locked/2\n* unlock_create_invalid/2\n* unlock_create/2\n* unlock_edit_not_locked/2\n* unlock_edit/2\n* unlock_edit_invalid/2\n* registration_create/2\n* registration_delete/2\n* invitation_create/2\n* invitation_resend/2\n* confirmation_create/2\n* confirmation_edit_invalid/2\n* confirmation_edit_expired/2\n* confirmation_edit/2\n* confirmation_edit_error/2\n\nYou can override any of the functions to customize the redirect path. Each\nfunction is passed the `conn` and `params` arguments from the controller.","ref":"Redirects.html","title":"Redirects","type":"behaviour"},{"doc":"use Redirects\n    import MyProject.Router.Helpers\n\n    # override the log out action back to the log in page\n    def session_delete(conn, _), do: redirect(conn, to: session_path(conn, :new))\n\n    # redirect the user to the login page after registering\n    def registration_create(conn, _), do: redirect(conn, to: session_path(conn, :new))\n\n    # disable the user_return_to feature on login\n    def session_create(conn, _), do: redirect(conn, to: landing_path(conn, :index))","ref":"Redirects.html#module-examples","title":"Examples - Redirects","type":"behaviour"},{"doc":"","ref":"Redirects.html#c:confirmation_create/2","title":"Redirects.confirmation_create/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:confirmation_edit/2","title":"Redirects.confirmation_edit/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:confirmation_edit_error/2","title":"Redirects.confirmation_edit_error/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:confirmation_edit_expired/2","title":"Redirects.confirmation_edit_expired/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:confirmation_edit_invalid/2","title":"Redirects.confirmation_edit_invalid/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:invitation_create/2","title":"Redirects.invitation_create/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:invitation_resend/2","title":"Redirects.invitation_resend/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:password_create/2","title":"Redirects.password_create/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:password_update/2","title":"Redirects.password_update/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:registration_create/2","title":"Redirects.registration_create/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:registration_delete/2","title":"Redirects.registration_delete/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:registration_update/3","title":"Redirects.registration_update/3","type":"callback"},{"doc":"","ref":"Redirects.html#c:session_create/2","title":"Redirects.session_create/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:session_delete/2","title":"Redirects.session_delete/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:unlock_create/2","title":"Redirects.unlock_create/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:unlock_create_invalid/2","title":"Redirects.unlock_create_invalid/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:unlock_create_not_locked/2","title":"Redirects.unlock_create_not_locked/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:unlock_edit/2","title":"Redirects.unlock_edit/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:unlock_edit_invalid/2","title":"Redirects.unlock_edit_invalid/2","type":"callback"},{"doc":"","ref":"Redirects.html#c:unlock_edit_not_locked/2","title":"Redirects.unlock_edit_not_locked/2","type":"callback"},{"doc":"","ref":"Responders.Html.html","title":"Responders.Html","type":"module"},{"doc":"","ref":"Responders.Json.html","title":"Responders.Json","type":"module"},{"doc":"# Coherence\n\n[![Build Status](https://travis-ci.org/smpallen99/coherence.svg?branch=master)](https://travis-ci.org/smpallen99/coherence) [![Hex Version][hex-img]][hex] [![License][license-img]][license]\n\n[hex-img]: https://img.shields.io/hexpm/v/coherence.svg\n[hex]: https://hex.pm/packages/coherence\n[license-img]: http://img.shields.io/badge/license-MIT-brightgreen.svg\n[license]: http://opensource.org/licenses/MIT\n\nCheckout the [Coherence Demo Project](https://github.com/smpallen99/coherence_demo) to see an example project using Coherence.\n\nCoherence is a full featured, configurable authentication system for Phoenix, with the following modules:\n\n* [Database Authenticatable](#authenticatable): handles hashing and storing an encrypted password in the database.\n* [Invitable](#invitable): sends invites to new users with a sign-up link, allowing the user to create their account with their own password.\n* [Registerable](#registerable): allows anonymous users to register a users email address and password.\n* [Confirmable](#confirmable): new accounts require clicking a link in a confirmation email.\n* [Recoverable](#recoverable): provides a link to generate a password reset link with token expiry.\n* [Trackable](#trackable): saves login statistics like login counts, timestamps, and IP address for each user.\n* [Lockable](#lockable): locks an account when a specified number of failed sign-in attempts has been exceeded.\n* [Unlockable With Token](#unlockable-with-token): provides a link to send yourself an unlock email.\n* [Rememberable](#remember-me): provides persistent login with 'Remember me?' check box on login page.\n\nCoherence provides flexibility by adding namespaced templates and views for only the options specified by the `mix coh.install` command. This boiler plate code is added to your `lib/my_project/web/templates/coherence` and `lib/my_project/web/views/coherence` directories.\n\nOnce the boilerplate has been generated, you are free to customize the source as required.\n\nAs well, a `lib/my_project/web/coherence_web.ex` is added. Migrations are also generated to add the required database fields.\n\nSee the [Docs](https://hexdocs.pm/coherence/Coherence.html) and [Wiki](https://github.com/smpallen99/coherence/wiki) for more information.","ref":"readme.html","title":"Coherence","type":"extras"},{"doc":"1. Add coherence to your list of dependencies in `mix.exs`:\n\n      ```elixir\n      def deps do\n        [{:coherence, \"~> 0.8\"}]\n      end\n      ```\n\n  2. Ensure coherence is started before your application:\n\n      ```elixir\n      def application do\n        extra_applications: [..., :coherence]]\n      end\n      ```","ref":"readme.html#installation","title":"Installation - Coherence","type":"extras"},{"doc":"After upgrading a Coherence version, you should generate the boilerplate files. To assist this process, use the `--reinstall` option.\n\nThis option uses your project's existing coherence config and runs the the installer with the same options.\n\n```shell\nmix coh.install --reinstall\n```\n\nRun a `git diff` to review the updated files. If you had updated any of the boilerplate files, you may need to manually integrate the changes into the newly generated files.\n\nRun `mix help coh.install` for more information.","ref":"readme.html#upgrading","title":"Upgrading - Coherence","type":"extras"},{"doc":"Coherence supports projects created with the older `mix phoenix.new` and the newer `mix phx.new` commands. Separate versions of the mix tasks exist for each project structure.\n\nFor projects created with `mix phx.new`, use the following mix tasks:\n\n* `coh.install`\n* `coh.clean`\n\nFor projects created with `mix phx.new --umbrella`, ensure you are in the app directory and use the following options for the install:\n* `cd apps/my_project`\n* `coh.install --web-module MyProjectWeb --web-path ../my_project_web/lib/my_project_web`","ref":"readme.html#phoenix-phx-project-structure","title":"Phoenix & Phx Project Structure - Coherence","type":"extras"},{"doc":"First, decide which modules you would like to use for your project. For the following example we are going to use a full install except for the confirmable option.\n\nRun the installer\n\n```bash\n$ mix coh.install --full-invitable\n```\n\nThis will:\n\n* add the coherence configuration to the end of your `config/config.exs` file.\n* add a new User model if one does not already exist\n* add migration files\n  * timestamp_add_coherence_to_user.exs if the User model already exists\n  * timestamp_create_coherence_user.exs if the User model does not exist\n  * timestamp_create_coherence_invitable.exs\n* add view files lib/my_project/web/views/coherence/\n* add template files to lib/my_project/web/templates/coherence\n* add email files to lib/my_project/web/emails/coherence\n* add lib/my_project/web/coherence_web.ex file\n* add lib/my_project/web/coherence_messages.ex file\n\nYou should review your `config/config.exs` as there are a couple items you will need to customize like email address and mail api_key. If you don't edit the email_from value to something different than its default, emails may not be sent.\n\nSee [Installer](#installer) for more install options.\n\nYou will need to update a few files manually.\n\n```elixir\n# lib/my_project_web/router.ex\n\ndefmodule MyProjectWeb.Router do\n  use MyProjectWeb, :router\n  use Coherence.Router         # Add this\n\n  pipeline :browser do\n    plug :accepts, [\"html\"]\n    plug :fetch_session\n    plug :fetch_flash\n    plug :protect_from_forgery\n    plug :put_secure_browser_headers\n    plug Coherence.Authentication.Session  # Add this\n  end\n\n  # Add this block\n  pipeline :protected do\n    plug :accepts, [\"html\"]\n    plug :fetch_session\n    plug :fetch_flash\n    plug :protect_from_forgery\n    plug :put_secure_browser_headers\n    plug Coherence.Authentication.Session, protected: true\n  end\n\n  # Add this block\n  scope \"/\" do\n    pipe_through :browser\n    coherence_routes()\n  end\n\n  # Add this block\n  scope \"/\" do\n    pipe_through :protected\n    coherence_routes :protected\n  end\n\n  scope \"/\", MyProjectWeb do\n    pipe_through :browser\n\n    get \"/\", PageController, :index\n    # add public resources below\n  end\n\n  scope \"/\", MyProjectWeb do\n    pipe_through :protected\n\n    # add protected resources below\n    resources \"/privates\", MyProjectWeb.PrivateController\n  end\nend\n```\n**Important**: Note the name-spacing above. Unless you generate coherence controllers, ensure that the scopes, `scope \"/\" do`, do not include your projects' scope here. If so, the coherence routes will not work!\n\nIf the installer created a user schema (one did not already exist), there is nothing you need to do with that generated file. Otherwise, update your existing schema (assuming its `Accounts.User` like this:\n\n```elixir\n# lib/my_project/accounts/user.ex\n\ndefmodule MyProject.Accounts.User do\n  use Ecto.Schema\n  use Coherence.Schema                                    # Add this\n\n  schema \"users\" do\n    field :name, :string\n    field :email, :string\n    coherence_schema()                                    # Add this\n\n    timestamps()\n  end\n\n  def changeset(model, params \\\\ %{}) do\n    model\n    |> cast(params, [:name, :email] ++ coherence_fields)  # Add this\n    |> validate_required([:name, :email])\n    |> validate_format(:email, ~r/@/)\n    |> validate_coherence(params)                         # Add this\n  end\n\n  def changeset(model, params, :password) do\n    model\n    |> cast(params, ~w(password password_confirmation reset_password_token reset_password_sent_at))\n    |> validate_coherence_password_reset(params)\n  end\nend\n```\n\nAn alternative approach is add the authentication plugs to individual controllers that require authentication. You will want to use this approach if you require authentication for a subset of actions in a controller.\n\nFor example, lets say you want to show a list of products for everyone visiting the site, but only want authenticated users to be able to create, update, and delete products. You could do the following:\n\nEnsure the following is in your `lib/my_project_web/router.ex` file:\n\n```elixir\nscope \"/\", MyProjectWeb do\n  pipe_through :browser\n  resources \"/products\", ProductController\nend\n```\n\nIn your product controller add the following:\n\n```elixir\ndefmodule MyProjectWeb.ProductController do\n  use MyProjectWeb, :controller\n\n  plug Coherence.Authentication.Session, [protected: true] when action != :index\n\n  # ...\nend\n```","ref":"readme.html#getting-started","title":"Getting Started - Coherence","type":"extras"},{"doc":"```elixir\n{:require_current_password, true}, # Current password is required when updating new password.\n{:reset_token_expire_days, 2},\n{:confirmation_token_expire_days, 5},\n{:allow_unconfirmed_access_for, 0},\n{:max_failed_login_attempts, 5},\n{:unlock_timeout_minutes, 20},\n{:unlock_token_expire_minutes, 5},\n{:rememberable_cookie_expire_hours, 2*24},\n{:forwarded_invitation_fields, [:email, :name]}\n{:allow_silent_password_recovery_for_unknown_user, false},\n{:password_hashing_alg, Comeonin.Bcrypt}\n```\n\nYou can override this default configs. For example: you can add the following codes inside `config/config.exs`\n\n```elixir\nconfig :coherence,\n  require_current_password: false,\n  max_failed_login_attempts: 3\n```","ref":"readme.html#default-configuration","title":"Default Configuration - Coherence","type":"extras"},{"doc":"Coherence supports custom routes for registration and login. These configurations can be set globally or scoped.\n\nWhich routes can be custom?\n\n```\n%{\n  registrations_new:  \"/registrations/new\",\n  registrations:      \"/registrations\",\n  passwords:          \"/passwords\",\n  confirmations:      \"/confirmations\",\n  unlocks:            \"/unlocks\",\n  invitations:        \"/invitations\",\n  invitations_create: \"/invitations/create\",\n  invitations_resend: \"/invitations/:id/resend\",\n  sessions:           \"/sessions\",\n  registrations_edit: \"/registrations/edit\"\n}\n```\n\nTo set them globally add the following to you configuration:\n\n\n\n```elixir\nconfig :coherence,\n  default_routes: %{\n    registrations_edit: \"/accounts/edit\", ...},\n```\n\nTo set them scoped for each mode (protected, public, etc..):\n\n```elixir\nscope \"/\" do\n  pipe_through :protected\n  coherence_routes :protected, [\n    custom_routes: %{registratons_edit: \"/accounts/edit\", ...}\n  ]\nend\n```","ref":"readme.html#custom-registration-and-sessions-routes","title":"Custom registration and sessions routes - Coherence","type":"extras"},{"doc":"Coherence supports channel authentication using `Phoenix.Token`. To enable channel authentication do the following:\n\nAdd the following option to coherence configuration.\n\n```elixir\nconfig :coherence,\n  user_token: true\n```\n\nUpdate your socket module:\n\n```elixir\ndefmodule MyProjectWeb.UserSocket do\n  use Phoenix.Socket\n\n  def connect(%{\"token\" => token}, socket) do\n    case Coherence.verify_user_token(socket, token, &assign/3) do\n      {:error, _} -> :error\n      {:ok, socket} -> {:ok, socket}\n    end\n  end\nend\n```","ref":"readme.html#phoenix-channel-authentication","title":"Phoenix Channel Authentication - Coherence","type":"extras"},{"doc":"Coherence supports `Gettext` for all User facing messages. Your project's `Gettext` module is used by default.\n\nAll Coherence messages use `dgettext` with the `\"coherence\"` domain. This means that after you run `mix gettext.extract`, you will see `coherence.pot` files generated. Running `mix gettext.merge priv/gettext` will generate the corresponding `coherence.po` files.\n\nCoherence does not come with pre-translated `.po` files. We figure that you will want to tweak the Coherence language to suite your application.\n\nAll files added to your project with the Coherence generators include a `import MyProjectWeb.Gettext` with the `\"coherence\"` domain.\n\nThe other messages that Coherence uses internal are pulled from the `my_project_web/coherence_messages.ex` file that is generated with the `coh.install` mix task. You can edit this file and customize it as required.\n\nTo assist in upgrades to future releases of Coherence, the `MyProject.Coherence.Messages` module uses the `Coherence.Messages` behaviour which defines each message function. So, if you miss a message after upgrading, you will see a message during compile time.","ref":"readme.html#localization-with-gettext","title":"Localization with Gettext - Coherence","type":"extras"},{"doc":"#","ref":"readme.html#option-overview","title":"Option Overview - Coherence","type":"extras"},{"doc":"Handles hashing and storing an encrypted password in the database.\n\nProvides `/sessions/new` and `/sessions/delete` routes for logging in and out with\nthe appropriate templates and view.\n\nThe following columns are added the ` _add_coherence_to_user.exs` migration:\n\n* :password_hash, :string - the encrypted password\n\n#","ref":"readme.html#authenticatable","title":"Authenticatable - Coherence","type":"extras"},{"doc":"Handles sending invites to new users with a sign-up link, allowing the user to create their account with their own password.\n\nProvides `/invitations/new` and `invitations/edit` routes for creating a new invitation and creating a new account from the invite email.\n\nThese routes can be configured to require login by using the `coherence_routes :private` macro in your router.exs file.\n\nInvitation token timeout will be added in the future.\n\nThe following table is created by the generated ` _create_coherence_invitable.exs` migration:\n\n```elixir\ncreate table(:invitations) do\n  add :name, :string\n  add :email, :string\n  add :token, :string\nend\n```\n\n#","ref":"readme.html#invitable","title":"Invitable - Coherence","type":"extras"},{"doc":"Allows anonymous users to register a users email address and password.\n\nProvides `/registrations/new`, `create`, `edit`, `update`, `show`, and `delete` routes for managing registrations.\n\nAdds the following:\n\n* `Register New Account` to the log-in page.\n* `Link to account page on layout helpers`\n* Show page with `edit` and `delete` links\n* `edit` page\n\nIt is recommended that the :confirmable option is used with :registerable to\nensure a valid email address is captured.\n\n#","ref":"readme.html#registerable","title":"Registerable - Coherence","type":"extras"},{"doc":"Requires a new account be confirmed. During registration, a confirmation token is generated and sent to the registering email. This link must be clicked before the user can sign-in.\n\nProvides `edit` action for the `/confirmations` route.\n\nThe confirmation token expiry default of 5 days can be changed with the `:confirmation_token_expire_days` config entry.\n\n#","ref":"readme.html#confirmable","title":"Confirmable - Coherence","type":"extras"},{"doc":"Allows users to reset their password using an expiring token send by email.\n\nProvides `new`, `create`, `edit`, `update` actions for the `/passwords` route.\n\nAdds a \"Forgot your password?\" link to the log-in form. When clicked, the user provides their email address and if found, sends a reset password instructions email with a reset link.\n\nThe expiry timeout can be changed with the `:reset_token_expire_days` config entry.\n\nBy default, providing an unknown email address will result in a form error. If you want to prevent that and display a confirmation message even if the email isn’t found, set the `:allow_silent_password_recovery_for_unknown_user` to `true`.\n\n#","ref":"readme.html#recoverable","title":"Recoverable - Coherence","type":"extras"},{"doc":"Saves login statistics like login counts, timestamps, and IP address for each user.\n\nAdds the following database field to your User model with the generated migration:\n\n```elixir\nadd :sign_in_count, :integer, default: 0  # how many times the user has logged in\nadd :current_sign_in_at, :datetime        # the current login timestamp\nadd :last_sign_in_at, :datetime           # the timestamp of the previous login\nadd :current_sign_in_ip, :string          # the current login IP adddress\nadd :last_sign_in_ip, :string             # the IP address of the previous login\n```\n\n#","ref":"readme.html#trackable","title":"Trackable - Coherence","type":"extras"},{"doc":"Locks an account when a specified number of failed sign-in attempts has been exceeded.\n\nThe following defaults can be changed with the following config entries:\n\n* `:unlock_timeout_minutes`\n* `:max_failed_login_attempts`\n\nAdds the following database field to your User model with the generated migration:\n\n```elixir\nadd :failed_attempts, :integer, default: 0\nadd :unlock_token, :string\nadd :locked_at, :datetime\n```\n\n#","ref":"readme.html#lockable","title":"Lockable - Coherence","type":"extras"},{"doc":"Provides a link to send yourself an unlock email. When the user clicks the link, the user is presented a form to enter their email address and password. If the token has not expired and the email and password are valid, a unlock email is sent to the user's email address with an expiring token.\n\nThe default expiry time can be changed with the `:unlock_token_expire_minutes` config entry.\n\n#","ref":"readme.html#unlockable-with-token","title":"Unlockable with Token - Coherence","type":"extras"},{"doc":"The `rememberable` option provides persistent login when the 'Remember Me?' box is checked during login.\n\nWith this feature, you will automatically be logged in from the same browser when your current login session dies using a configurable expiring persistent cookie.\n\nFor security, both a token and series number stored in the cookie on initial login. Each new creates a new token, but preserves the series number, providing protection against fraud. As well, both the token and series numbers are hashed before saving them to the database, providing protection if the database is compromised.\n\nThe following defaults can be changed with the following config entries:\n\n* :rememberable_cookie_expire_hours (2\\*24)\n* :login_cookie                     (\"coherence_login\")\n\nThe following table is created by the generated ` _create_coherence_rememberable.exs` migration:\n\n```elixir\ncreate table(:rememberables) do\n  add :series_hash, :string\n  add :token_hash, :string\n  add :token_created_at, :datetime\n  add :user_id, references(:users, on_delete: :delete_all)\n\n  timestamps\nend\n\ncreate index(:rememberables, [:user_id])\ncreate index(:rememberables, [:series_hash])\ncreate index(:rememberables, [:token_hash])\ncreate unique_index(:rememberables, [:user_id, :series_hash, :token_hash])\n```\n\nThe `--rememberable` install option is not provided in any of the installer group options. You must provide the `--rememberable` option to install the migration and its support.\n\n#","ref":"readme.html#remember-me","title":"Remember Me - Coherence","type":"extras"},{"doc":"The `--user-active-field` option can be given to the installer to support inactive users. When a user is set as inactive, they will not be able to login.\n\nUsing this option adds a `active` field to the user schema with a default of `true`.","ref":"readme.html#user-active-field","title":"User Active Field - Coherence","type":"extras"},{"doc":"#","ref":"readme.html#mix-tasks","title":"Mix Tasks - Coherence","type":"extras"},{"doc":"The following examples illustrate various configuration scenarios for the install mix task:\n\n```bash\n# Install with only the `authenticatable` option\n$ mix coh.install\n\n# Install all the options except `confirmable` and `invitable`\n$ mix coh.install --full\n\n# Install all the options except `invitable`\n$ mix coh.install --full-confirmable\n\n# Install all the options except `confirmable`\n$ mix coh.install --full-invitable\n\n# Install the `full` options except `lockable` and `trackable`\n$ mix coh.install --full --no-lockable --no-trackable\n```\n\nAnd some reinstall examples:\n\n```bash\n# Reinstall with defaults (--silent --no-migrations --no-config --confirm-once)\n$ mix coh.install --reinstall\n\n# Confirm to overwrite files, show instructions, and generate migrations\n$ mix coh.install --reinstall --no-confirm-once --with-migrations\n```\n\nRun `$ mix help coh.install` for more information.\n\n#","ref":"readme.html#installer","title":"Installer - Coherence","type":"extras"},{"doc":"The following examples illustrate how to remove the files created by the installer:\n\n```bash\n# Clean all the installed files\n$ mix coh.clean --all\n\n# Clean only the installed view and template files\n$ mix coh.clean --views --templates\n\n# Clean all but the models\n$ mix coh.clean --all --no-models\n\n# Prompt once to confirm the removal\n$ mix coh.clean --all --confirm-once\n```\n\nAfter installation, if you later want to remove one more options, here are a couple examples:\n\n```bash\n# Clean one option\n$ mix coh.clean --options=recoverable\n\n# Clean several options without confirmation\n$ mix coh.clean --no-confirm --options=\"recoverable unlockable-with-token\"\n\n# Test the uninstaller without removing files\n$ mix coh.clean --dry-run --options=\"recoverable unlockable-with-token\"\n```","ref":"readme.html#clean","title":"Clean - Coherence","type":"extras"},{"doc":"The `coh.install` mix task generates a bunch of boiler plate code so you can easily customize the views, templates, and mailer.\n\nAlso, checkout the Coherence.Config module for a list of config items you can use to tune the behaviour of Coherence.\n\n#","ref":"readme.html#customization","title":"Customization - Coherence","type":"extras"},{"doc":"By default, controller boilerplate is not generated. To add controllers, use the controller generators.\n\n* For phx projects, use the `mix coh.gen.controllers` task.\n\nThe generated controllers are named `MyProjectWeb.Coherence.SessionController` as an example. Generated controllers are located in `lib/my_project_web/controllers/coherence/`\n\nIf the controllers are generated, you will need to change your router to use the new names. For example:\n\n```elixir\n# lib/my_project_web/router.ex\n\ndefmodule MyProjectWeb.Router do\n  use MyProjectWeb, :router\n  use Coherence.Router\n\n  # ...\n\n  scope \"/\", MyProjectWeb do   # note the addition of MyProjectWeb\n    pipe_through :public\n    coherence_routes()\n  end\n\n  scope \"/\", MyProjectWeb do   # note the addition of MyProjectWeb\n    pipe_through :browser\n    coherence_routes :protected\n  end\n\n  # ...\nend\n```\n\nAs of Coherence v0.6.0, the generated controller modules have very little code. All the controller actions and helper functions are included with a `use xxxControllerBase` call.\n\nTo change an action, simply define the appropriate function and add your own implementation. You may want to copy over the implementation found in your projects `deps/coherence/lib/coherence/controllers/xxx_controller_base.ex` file as a starting point and customize it as needed.\n\nAlternatively, if can create the action, handle some specify behavior and call `super(conn, params)` so invoke the default action. Depending on your customization, this may be the best approach since its easier to see how changes when upgrading to new versions of Coherence.\n\n#","ref":"readme.html#custom-controllers","title":"Custom Controllers - Coherence","type":"extras"},{"doc":"By default, Coherence assumes you want all available routes for the `opts` you've configured. However, you can specify which routes should be available by modifying your configuration.\n\nFor example, if you want all of the routes for `authenticatable`, but only the `new` and `create` actions from `registerable`:\n\n```elixir\n# config/config.exs\nconfig :coherence,\n  # ...\n  opts: [:authenticatable, registerable: [:new, :create]]\n```\n\n#","ref":"readme.html#customizing-routes","title":"Customizing Routes - Coherence","type":"extras"},{"doc":"Many of the controller actions redirect the user after create and update actions. These redirections can be customized by adding function call backs in the `lib/my_project_web/controllers/coherence/redirect.ex` module that is generated by the `mix coh.install` task.\n\nFor example, to have the user redirected to the login screen after logging out at the following:\n\n```elixir\ndefmodule Coherence.Redirects do\n  use Redirects\n  import MyProjectWeb.Router.Helpers\n\n  # override the log out action back to the log in page\n  def session_delete(conn, _), do: redirect(conn, session_path(conn, :new))\nend\n```\n\nSee the documentation for further details.\n\n#","ref":"readme.html#customizing-redirections","title":"Customizing Redirections - Coherence","type":"extras"},{"doc":"To customize how application responds to html or json format, you can override methods in the `lib/my_project_web/controllers/coherence/responders/html.ex` or `lib/my_project_web/controllers/coherence/responders/json.ex`.\n\n#","ref":"readme.html#customizing-responders","title":"Customizing Responders - Coherence","type":"extras"},{"doc":"By default coherence uses its own layout which is installed to `lib/my_project_web/templates/coherence/layout/app.html.eex`.\n\nIf you want to customize coherence controllers layout, you can follow different approaches:\n\n* Edit the generated layout at `lib/my_project_web/templates/coherence/layout/app.html.eex`.\n* Set `:layout` in your config file. e.g. `config :coherence, :layout, {MyProjectWeb.LayoutView, :app}`\n* Install coherence controllers to application and edit them, to use layout module different from `Coherence.LayoutView`\n* And the last solution is to use `plug :put_layout` in your `lib/my_project_web/router.ex` file. For example:\n\n```elixir\ndefmodule MyProjectWeb.Router do\n  # ...\n  pipeline :coherence do\n    plug :put_layout, {MyProjectWeb.LayoutView, :app}\n  end\n  # ...\n  scope \"/\" do\n    pipe_through [:protected, :coherence]\n    coherence_routes :protected\n  end\n  #...\nend\n```","ref":"readme.html#customizing-layout","title":"Customizing layout - Coherence","type":"extras"},{"doc":"The User model changeset used by Coherence can be customized for each Coherence controller. To customize the changeset, set the `changeset` config option.\n\nFor example, the following defines a changeset/3 function in your user model:\n\n```elixir\n# config/config.exs\nconfig :coherence,\n  # ...\n  changeset: {MyProject.User, :changeset}\n```\n\nNow add a new `changeset/3` function to the user model. The following example defines a custom changeset for the registration controller:\n\n```elixir\n# lib/coherence/coherence/user.ex\ndefmodule CoherenceDemo.User do\n  use Ecto.Schema\n  use Coherence.Schema\n\n  # ...\n\n  def changeset(model, params \\\\ %{}) do\n    model\n    |> cast(params, [:name, :email] ++ coherence_fields)\n    |> validate_required([:name, :email])\n    |> validate_format(:email, ~r/@/)\n    |> unique_constraint(:email)\n    |> validate_coherence(params)\n  end\n  def changeset(model, params, :registration) do\n    # custom changeset  for registration controller\n    model\n    |> cast(params, [:name, :email] ++ coherence_fields)\n    |> validate_required([:name, :email])\n    |> validate_format(:email, ~r/@/)\n    |> unique_constraint(:email)\n    |> validate_coherence(params)\n  end\n  def changeset(model, params, _which) do\n    # use the default changeset for all other coherence controllers\n    changeset model, params\n  end\nend\n```\n\nWhen a custom changeset is configured, the changeset function is called with an atom indicating the controller calling the changeset, allowing you to match on specific controllers.\n\nThe list of controller actions are:\n\n* :confirmation\n* :invitation\n* :password\n* :registration\n* :session\n* :unlock","ref":"readme.html#customizing-user-changeset","title":"Customizing User Changeset - Coherence","type":"extras"},{"doc":"Coherence uses the `Bcrypt` algorithm by default for hashing passwords. However, with the update to Comeonin 4.0, you can now change the hashing algorithm.\n\nComeonin currently supports the following 3 algorithms:\n\n* [Argon2](https://github.com/riverrun/argon2_elixir)\n* [Bcrypt](https://github.com/riverrun/bcrypt_elixir)\n* [Pbkdf2](https://github.com/riverrun/pbkdf2_elixir)\n\n#","ref":"readme.html#customizing-password-hashing-algorithm","title":"Customizing Password Hashing Algorithm - Coherence","type":"extras"},{"doc":"To change the default in an existing project (to Argon2 for example), make the following 2 changes:\n\n* Edit your `config/config.exs` file add/change the following line:\n\n```elixir\n# config/config.exs\nconfig :coherence,\n  # ...\n  password_hashing_alg: Comeonin.Argon2,\n  # ...\n```\n\n* add the dependency to `mix.exs`\n\n```elixir\n  # mix.exs\n  defp deps do\n    [\n      # ...\n      {:argon2_elixir, \"~> 1.3\"}\n    ]\n  end\n```\n\n#","ref":"readme.html#change-the-hashing-algorithm-in-an-existing-project","title":"Change the Hashing Algorithm in an Existing Project - Coherence","type":"extras"},{"doc":"To install Coherence in a new project with the `Pbkdf2` hashing algorithm (with the --full option for example):\n\n```bash\n# mix coh.install --full --password-hashing-alg=Comeonin.Argon2\n```\n\nand add the dependency\n\n```elixir\n  # mix.exs\n  defp deps do\n    [\n      # ...\n      {:pbkdf2_elixir, \"~> 0.12\"}\n    ]\n  end\n```\n\n#","ref":"readme.html#change-the-hashing-algorithm-in-an-existing-project","title":"Change the Hashing Algorithm in an Existing Project - Coherence","type":"extras"},{"doc":"The default hashing algorithms are setup for production use. They are very slow by design which can cause very slow tests and database seeding in the dev and test environments. To speed this up, you can add the following to you `config/dev.exs` and/or `config/test.exs` configuration.\n\nHowever, *DON'T  USE THESE SETTINGS IN PRODUCTION*\n\n```elixir\n# config/test.exs\nconfig :argon2_elixir,\n  t_cost: 1,\n  m_cost: 8\nconfig :bcrypt_elixir, log_rounds: 4\nconfig :pbkdf2_elixir, rounds: 1\n```\n\nNote: Only configure the algorithm you have configured!","ref":"readme.html#speed-up-tests-and-database-seeding-of-users","title":"Speed up Tests and Database Seeding of Users - Coherence","type":"extras"},{"doc":"During login, a current version of the user model is cashed in the credential store. During each authentication request, the user model is fetched from the credential store and placed in conn.assigns[:current_user] to avoid a database fetch on each request.\n\nYou can access the current user's name in a template like this:\n\n```elixir\n \n```\n\nAny of the user model's available data can be accessed this way.","ref":"readme.html#accessing-the-currently-logged-in-user","title":"Accessing the Currently Logged In User - Coherence","type":"extras"},{"doc":"If the user model is changed after login, a call to `update_login` must be done to update the credential store. For example, in your controller update function, call:\n\n```elixir\nCoherence.update_user_login(conn, user)\n```\n\nto update the credential store.\n\nThis is not needed for registration update page.","ref":"readme.html#updating-the-user-model","title":"Updating the User Model - Coherence","type":"extras"},{"doc":"The following configuration must be setup to send emails:\n\n```elixir\nconfig :coherence,\n  email_from_name: \"Some Name\",\n  email_from_email: \"myname@domain.com\"\n\nconfig :coherence, CoherenceDemoWeb.Coherence.Mailer,\n  adapter: Swoosh.Adapters.Sendgrid,\n  api_key: \"SENDGRID_API_KEY\"\n```\n\nYou may want to configure the email system to use system environment variables.\n\n```elixir\nconfig :coherence,\n  email_from_name: {:system, \"NAME\"},\n  email_from_email: {:system, \"EMAIL\"}\n\nconfig :coherence, CoherenceDemoWeb.Coherence.Mailer,\n  api_key: {:system, \"SENDGRID_API_KEY\"}\n```","ref":"readme.html#configuring-the-swoosh-email-adapter","title":"Configuring the Swoosh Email Adapter - Coherence","type":"extras"},{"doc":"For security, Coherence restricts what fields of a schema can be created or\nupdated with several configurable permitted attributes settings. When installing\na new project these settings are generated in the `:coherence` Config block.\n\nIn the case of an upgrade, the following values will be used when the configuration\nis not found:\n\n| Configuration Field | Defaults |\n| --- | --- |\n| `:registration_permitted_attributes` | [\"email\",\"name\",\"password\", \"current_password\", \"password_confirmation\"] |\n| `:invitation_permitted_attributes` | [\"name\",\"email\"] |\n| `:password_reset_permitted_attributes` | [\"reset_password_token\",\"password\", \"password_confirmation\"] |\n| `:session_permitted_attributes` | [\"remember\",\"email\",\"password\"] |","ref":"readme.html#permitted-attributes","title":"Permitted Attributes - Coherence","type":"extras"},{"doc":"Currently Coherence supports three modes of authentication including HTTP Basic, Session, and Token authentication.\n\nFor HTTP Basic and Token authentication, you will need to add the credentials into the Credential Store. This is not required for Session or IpAddress Authentication.\n\nIpAddress authentication is a good solution for server to server rest APIs.\n\n#","ref":"readme.html#authentication","title":"Authentication - Coherence","type":"extras"},{"doc":"```elixir\ncreds = Coherence.Authentication.Basic.encode_credentials(\"Admin\", \"SecretPass\")\nCoherence.CredentialStore.Server.put_credentials(creds, %{id: \"USER_ID_HERE\", role: :admin})\n```\n\n#","ref":"readme.html#add-http-basic-credentials-example","title":"Add HTTP Basic Credentials Example - Coherence","type":"extras"},{"doc":"```elixir\ntoken = Coherence.Authentication.Token.generate_token\nCoherence.CredentialStore.Server.put_credentials(token, %{id: \"USER_ID_HERE\", role: :admin})\n```\n\n#","ref":"readme.html#add-token-credentials-example","title":"Add Token Credentials Example - Coherence","type":"extras"},{"doc":"```elixir\nCoherence.CredentialStore.Server.put_credentials({127,0,0,1}, %{id: \"USER_ID_HERE\", role: :admin})\n```\n\nIpAddress authentication does not require this step. Its optional. If the user_data\nis not found in the credential store, the conn.assigns will not be set.\n\nTo add authentication, use on of the following three:\n\n#","ref":"readme.html#add-ip-credentials-example","title":"Add IP Credentials Example - Coherence","type":"extras"},{"doc":"```elixir\nplug Coherence.Authentication.Basic, realm: \"Secret\"\n```\n\nThe realm parameter is optional and can be omitted. By default \"Restricted Area\" will be used as realm name. You can also pass the error parameter, which should be a string or a function. If a string is passed, that string will be sent instead of the default message \"HTTP Authentication Required\" on authentication failure (with status code 401). If a function is passed, that function will be called with one argument, `conn`.\n\n#","ref":"readme.html#http-basic-plug-example","title":"HTTP Basic Plug Example - Coherence","type":"extras"},{"doc":"```elixir\nplug Coherence.Authentication.Token, source: :params, param: \"auth_token\", error: ~s'{\"error\":\"authentication required\"}'\n```\n\nThe error parameter is optional and is treated as in the example above. The source parameter defines how to retrieve the token from the connection. Currently, the three acceptable values are: `:params`, `:header` and `:session`. Their name is self-explainatory. The param parameter defines the name of the parameter/HTTP header/session key where the token is stored. This should cover most cases, but if retrieving the token is more complex than that, you can pass a tuple for the source parameter. The tuple must be in the form `{MyModule, :my_function, [\"param1\", 42]}`. The function must accept a connection as its first argument (which will be injected as the head of the given parameter list) and any other number of parameters, which must be given in the third element of the tuple. If no additional arguments are needed, an empty list must be given.\n\n#","ref":"readme.html#token-plug-example","title":"Token Plug Example - Coherence","type":"extras"},{"doc":"```elixir\nplug Coherence.Authentication.Session, cookie_expire: 10*60*60, login: &MyController.login/1, assigns_key: :authenticated_user\n```\n\nThe `:cookie_expire` value the expire time in seconds. The `:login` is a fun that will be passed `conn` if the user is not logged in. Use the `:assigns_key` to change the default `:current_user` value.\n\nNote that if you provide a login callback, that you must return `halt conn` a the end of the function.\n\n#","ref":"readme.html#session-plug-example","title":"Session Plug Example - Coherence","type":"extras"},{"doc":"```elixir\nplug Coherence.Authentication.IpAddress, allow: ~w(127.0.0.1 192.168.1.0/24)\nplug Coherence.Authentication.IpAddress, allow: ~w(0.0.0.0/0), deny: ~w(127.0.0.1)\n```\n\nThe first example will allow local host and any ip address in the subnet 192.168.1.0/255.255.255.0\n\nThe second example allows any ip except for localhost.","ref":"readme.html#ip-address-plug-example","title":"IP Address Plug Example - Coherence","type":"extras"},{"doc":"Coherence is a user management and authentication solution. Support for authorization (access control) can be achieved using another package like [Canary](https://github.com/cpjk/canary).\n\nFor an example of using [Canary](https://github.com/cpjk/canary) with Coherence, please visit the [CoherenceDemo canary branch](https://github.com/smpallen99/coherence_demo/tree/canary).","ref":"readme.html#authorization","title":"Authorization - Coherence","type":"extras"},{"doc":"We appreciate any contribution to Coherence. Check our [CODE_OF_CONDUCT.md](CODE_OF_CONDUCT.md) and [CONTRIBUTING.md](CONTRIBUTING.md) guides for more information. We usually keep a list of features and bugs [in the issue tracker][1].","ref":"readme.html#contributing","title":"Contributing - Coherence","type":"extras"},{"doc":"* Detailed Example [Coherence Demo](https://github.com/smpallen99/coherence_demo)\n* [Docs](https://hexdocs.pm/coherence/)\n\n  [1]: https://github.com/smpallen99/coherence/issues","ref":"readme.html#references","title":"References - Coherence","type":"extras"},{"doc":"`coherence` is Copyright (c) 2016-2018 E-MetroTel\n\nThe source is released under the MIT License.\n\nCheck [LICENSE](LICENSE) for more information.\n\nMuch of the authentication plugs code was taken from [PlugAuth](https://github.com/bitgamma/plug_auth), Copyright (c) 2014, Bitgamma OÜ","ref":"readme.html#license","title":"License - Coherence","type":"extras"},{"doc":"# Contributor Code of Conduct\n\nAs contributors and maintainers of this project, and in the interest of fostering an open and welcoming community, we pledge to respect all people who contribute through reporting issues, posting feature requests, updating documentation, submitting pull requests or patches, and other activities.\n\nWe are committed to making participation in this project a harassment-free experience for everyone, regardless of level of experience, gender, gender identity and expression, sexual orientation, disability, personal appearance, body size, race, ethnicity, age, religion, or nationality.\n\nExamples of unacceptable behavior by participants include:\n\n* The use of sexualized language or imagery\n* Personal attacks\n* Trolling or insulting/derogatory comments\n* Public or private harassment\n* Publishing other's private information, such as physical or electronic addresses, without explicit permission\n* Other unethical or unprofessional conduct.\n\nProject maintainers have the right and responsibility to remove, edit, or reject comments, commits, code, wiki edits, issues, and other contributions that are not aligned to this Code of Conduct. By adopting this Code of Conduct, project maintainers commit themselves to fairly and consistently applying these principles to every aspect of managing this project. Project maintainers who do not follow or enforce the Code of Conduct may be permanently removed from the project team.\n\nThis code of conduct applies both within project spaces and in public spaces when an individual is representing the project or its community.\n\nInstances of abusive, harassing, or otherwise unacceptable behavior may be reported by opening an issue or contacting one or more of the project maintainers.\n\nThis Code of Conduct is adapted from the [Contributor Covenant](http://contributor-covenant.org), version 1.2.0, available at [http://contributor-covenant.org/version/1/2/0/](http://contributor-covenant.org/version/1/2/0/)","ref":"code_of_conduct.html","title":"Contributor Code of Conduct","type":"extras"},{"doc":"# Contributing to Coherence\n\nPlease take a moment to review this document in order to make the contribution process easy and effective for everyone involved! Also make sure you read our [Code of Conduct](CODE_OF_CONDUCT.md) that outlines our commitment towards an open and welcoming environment.","ref":"contributing.html","title":"Contributing to Coherence","type":"extras"},{"doc":"Use the issues tracker for:\n\n* [bug reports](#bug-reports)\n* [submitting pull requests](#pull-requests)\n\nWe do our best to keep the issue tracker tidy and organized, making it useful for everyone. For example, we classify open issues per perceived difficulty, making it easier for developers to [contribute to Coherence](#pull-requests).","ref":"contributing.html#using-the-issue-tracker","title":"Using the issue tracker - Contributing to Coherence","type":"extras"},{"doc":"A bug is a _demonstrable problem_ that is caused by the code in the repository.\nGood bug reports are extremely helpful - thank you!\n\nGuidelines for bug reports:\n\n1. **Use the GitHub issue search** &mdash; check if the issue has already been\n   reported.\n\n2. **Check if the issue has been fixed** &mdash; try to reproduce it using the\n   `master` branch in the repository.\n\n3. **Isolate and report the problem** &mdash; ideally create a reduced test\n   case.\n\nPlease try to be as detailed as possible in your report. Include information about your Operating System, as well as your Erlang, Elixir, Phoenix, and Coherence versions. Please provide steps to reproduce the issue as well as the outcome you were expecting! All these details will help developers to fix any potential bugs.\n\nExample:\n\n> Short and descriptive example bug report title\n>\n> A summary of the issue and the environment in which it occurs. If suitable,\n> include the steps required to reproduce the bug.\n>\n> 1. This is the first step\n> 2. This is the second step\n> 3. Further steps, etc.\n>\n> ` ` - a link to the reduced test case (e.g. a GitHub Gist)\n>\n> Any other information you want to share that is relevant to the issue being\n> reported. This might include the lines of code that you have identified as\n> causing the bug, and potential solutions (and your opinions on their\n> merits).","ref":"contributing.html#bug-reports","title":"Bug reports - Contributing to Coherence","type":"extras"},{"doc":"Feature requests are welcome and should be discussed with the issue tracker. But take a moment to find out whether your idea fits with the scope and aims of the project. It's up to *you* to make a strong case to convince the community of the merits of this feature. Please provide as much detail and context as possible.","ref":"contributing.html#feature-requests","title":"Feature requests - Contributing to Coherence","type":"extras"},{"doc":"Code documentation (`@doc`, `@moduledoc`, `@typedoc`) has a special convention:\nthe first paragraph is considered to be a short summary.\n\nFor functions, macros and callbacks say what it will do. For example write something like:\n\n```elixir\n@doc \"\"\"\nMarks the given value as HTML safe.\n\"\"\"\ndef safe({:safe, value}), do: {:safe, value}\n```\n\nFor modules, protocols and types say what it is. For example write something like:\n\n```elixir\ndefmodule Phoenix.HTML do\n  @moduledoc \"\"\"\n  Conveniences for working HTML strings and templates.\n  ...\n  \"\"\"\n```\n\nKeep in mind that the first paragraph might show up in a summary somewhere, long texts in the first paragraph create very ugly summaries. As a rule of thumb anything longer than 80 characters is too long.\n\nTry to keep unnecessary details out of the first paragraph, it's only there to give a user a quick idea of what the documented \"thing\" does/is. The rest of the documentation string can contain the details, for example when a value and when `nil` is returned.\n\nIf possible include examples, preferably in a form that works with doctests. This makes it easy to test the examples so that they don't go stale and examples are often a great help in explaining what a function does.","ref":"contributing.html#contributing-documentation","title":"Contributing Documentation - Contributing to Coherence","type":"extras"},{"doc":"Good pull requests - patches, improvements, new features - are a fantastic help. They should remain focused in scope and avoid containing unrelated commits.\n\n**IMPORTANT**: By submitting a patch, you agree that your work will be licensed under the license used by the project.\n\nIf you have any large pull request in mind (e.g. implementing features, refactoring code, etc), **please ask first** otherwise you risk spending a lot of time working on something that the project's developers might not want to merge into the project.\n\nPlease adhere to the coding conventions in the project (indentation, accurate comments, etc.) and don't forget to add your own tests and documentation. When working with git, we recommend the following process in order to craft an excellent pull request:\n\n1. [Fork](http://help.github.com/fork-a-repo/) the project, clone your fork,\n   and configure the remotes:\n\n   ```bash\n   # Clone your fork of the repo into the current directory\n   git clone https://github.com/ /coherence\n   # Navigate to the newly cloned directory\n   cd coherence\n   # Assign the original repo to a remote called \"upstream\"\n   git remote add upstream https://github.com/smpallen99/coherence\n   ```\n\n2. Pick a project to interactively see your Coherence changes:\n\n  * Create your own project or use the [Coherence Demo](https://github.com/smpallen99/coherence_demo)\n\n  * Change the :coherence dependency in your project to a path directive like\n\n    ```elixir\n      def deps do\n        [\n          {:coherence, path: \"../coherence\"}\n        ]\n      end\n    ```\n\n  * Get the new deps and remove the old build dir\n\n    ```bash\n    mix deps.get coherence\n    rm -rf build/dev/lib/coherence\n    ```\n\n  * Now your project should be setup to your up your local changes every time you compile your project.\n\n3. If you cloned a while ago, get the latest changes from upstream, and update your fork:\n\n   ```bash\n   git checkout master\n   git pull upstream master\n   git push\n   ```\n\n4. Create a new topic branch (off of `master`) to contain your feature, change,\n   or fix.\n\n   **IMPORTANT**: Making changes in `master` is discouraged. You should always\n   keep your local `master` in sync with upstream `master` and make your\n   changes in topic branches.\n\n   ```bash\n   git checkout -b  \n   ```\n\n5. Commit your changes in logical chunks. Keep your commit messages organized,\n   with a short description in the first line and more detailed information on\n   the following lines. Feel free to use Git's\n   [interactive rebase](https://help.github.com/articles/interactive-rebase)\n   feature to tidy up your commits before making them public.\n\n6. Make sure all the tests are still passing.\n\n   ```bash\n   mix test\n   ```\n\n7. Push your topic branch up to your fork:\n\n   ```bash\n   git push origin  \n   ```\n8. If you modified the controllers in `web/controllers/` you need\n   to run a mix task to copy those files to `priv/templates/coherence.install/controllers`.\n\n   ```bash\n   mix coherence.make_templates\n   ```\n\n9. [Open a Pull Request](https://help.github.com/articles/using-pull-requests/)\n    with a clear title and description.\n\n10. If you haven't updated your pull request for a while, you should consider\n   rebasing on master and resolving any conflicts.\n\n   **IMPORTANT**: _Never ever_ merge upstream `master` into your branches. You\n   should always `git rebase` on `master` to bring your changes up to date when\n   necessary.\n\n   ```bash\n   git checkout master\n   git pull upstream master\n   git checkout  \n   git rebase master\n   ```\n\nThank you for your contributions!","ref":"contributing.html#pull-requests","title":"Pull requests - Contributing to Coherence","type":"extras"},{"doc":"The MIT License (MIT)\n\nCopyright (c) 2016-2023 E-MetroTel\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"}]}